@@ -4,18 +4,17 @@
 
 #include "str.h"
 #include "loglib.h"
+#include "types.h"
 
 
 
-struct call;
-struct stream_fd;
 struct ice_agent;
 enum log_format;
 
 struct log_info {
 	union {
-		struct call *call;
-		struct stream_fd *stream_fd;
+		call_t *call;
+		stream_fd *stream_fd;
 		const str *str;
 		const char *cstr;
 		struct ice_agent *ice_agent;
