@@ -1,6 +1,7 @@
 #ifdef HAVE_MQTT
 
 #include "mqtt.h"
+
 #include <mosquitto.h>
 #include <errno.h>
 #include <string.h>
@@ -8,6 +9,7 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <json-glib/json-glib.h>
+
 #include "main.h"
 #include "log.h"
 #include "log_funcs.h"
@@ -152,7 +154,7 @@ void mqtt_publish(char *s) {
 }
 
 
-static void mqtt_call_stats(struct call *call, JsonBuilder *json) {
+static void mqtt_call_stats(call_t *call, JsonBuilder *json) {
 	json_builder_set_member_name(json, "call_id");
 	json_builder_add_string_value(json, call->callid.s);
 }
@@ -179,7 +181,7 @@ static void mqtt_monologue_stats(struct call_monologue *ml, JsonBuilder *json) {
 		json_builder_set_member_name(json, "duration");
 		json_builder_add_int_value(json, mp->coder.duration);
 		json_builder_set_member_name(json, "repeat");
-		json_builder_add_int_value(json, mp->repeat);
+		json_builder_add_int_value(json, mp->opts.repeat);
 		json_builder_set_member_name(json, "frame_time");
 		json_builder_add_int_value(json, mp->last_frame_ts);
 
@@ -214,7 +216,7 @@ static void mqtt_ssrc_stats(struct ssrc_ctx *ssrc, JsonBuilder *json, struct cal
 	mutex_unlock(&ssrc->tracker.lock);
 
 	unsigned int clockrate = 0;
-	struct rtp_payload_type *pt = g_hash_table_lookup(media->codecs.codecs, GUINT_TO_POINTER(prim_pt));
+	rtp_payload_type *pt = t_hash_table_lookup(media->codecs.codecs, GUINT_TO_POINTER(prim_pt));
 	if (pt) {
 		json_builder_set_member_name(json, "codec");
 		json_builder_add_string_value(json, pt->encoding.s);
@@ -239,8 +241,8 @@ static void mqtt_ssrc_stats(struct ssrc_ctx *ssrc, JsonBuilder *json, struct cal
 
 	// copy out values
 	int64_t packets, octets, packets_lost, duplicates;
-	packets = atomic64_get(&ssrc->packets);
-	octets = atomic64_get(&ssrc->octets);
+	packets = atomic64_get_na(&ssrc->stats->packets);
+	octets = atomic64_get_na(&ssrc->stats->bytes);
 	packets_lost = sc->packets_lost;
 	duplicates = sc->duplicates;
 
@@ -327,18 +329,18 @@ static void mqtt_ssrc_stats(struct ssrc_ctx *ssrc, JsonBuilder *json, struct cal
 
 static void mqtt_stream_stats_dir(const struct stream_stats *s, JsonBuilder *json) {
 	json_builder_set_member_name(json, "bytes");
-	json_builder_add_int_value(json, atomic64_get(&s->bytes));
+	json_builder_add_int_value(json, atomic64_get_na(&s->bytes));
 	json_builder_set_member_name(json, "packets");
-	json_builder_add_int_value(json, atomic64_get(&s->packets));
+	json_builder_add_int_value(json, atomic64_get_na(&s->packets));
 	json_builder_set_member_name(json, "errors");
-	json_builder_add_int_value(json, atomic64_get(&s->errors));
+	json_builder_add_int_value(json, atomic64_get_na(&s->errors));
 }
 
 
 static void mqtt_stream_stats(struct packet_stream *ps, JsonBuilder *json) {
 	mutex_lock(&ps->in_lock);
 
-	struct stream_fd *sfd = ps->selected_sfd;
+	stream_fd *sfd = ps->selected_sfd;
 	if (sfd) {
 		json_builder_set_member_name(json, "address");
 		json_builder_add_string_value(json, sockaddr_print_buf(&sfd->socket.local.address));
@@ -349,7 +351,7 @@ static void mqtt_stream_stats(struct packet_stream *ps, JsonBuilder *json) {
 
 	json_builder_set_member_name(json, "ingress");
 	json_builder_begin_object(json);
-	mqtt_stream_stats_dir(&ps->stats_in, json);
+	mqtt_stream_stats_dir(ps->stats_in, json);
 
 	json_builder_set_member_name(json, "SSRC");
 	json_builder_begin_array(json);
@@ -370,7 +372,7 @@ static void mqtt_stream_stats(struct packet_stream *ps, JsonBuilder *json) {
 
 	json_builder_set_member_name(json, "egress");
 	json_builder_begin_object(json);
-	mqtt_stream_stats_dir(&ps->stats_out, json);
+	mqtt_stream_stats_dir(ps->stats_out, json);
 
 	json_builder_set_member_name(json, "SSRC");
 	json_builder_begin_array(json);
@@ -390,8 +392,6 @@ static void mqtt_stream_stats(struct packet_stream *ps, JsonBuilder *json) {
 
 
 static void mqtt_media_stats(struct call_media *media, JsonBuilder *json) {
-	media_update_stats(media);
-
 	json_builder_set_member_name(json, "media_index");
 	json_builder_add_int_value(json, media->index);
 
@@ -426,7 +426,7 @@ static void mqtt_media_stats(struct call_media *media, JsonBuilder *json) {
 }
 
 
-static void mqtt_full_call(struct call *call, JsonBuilder *json) {
+static void mqtt_full_call(call_t *call, JsonBuilder *json) {
 	rwlock_lock_r(&call->master_lock);
 
 	log_info_call(call);
@@ -436,7 +436,7 @@ static void mqtt_full_call(struct call *call, JsonBuilder *json) {
 	json_builder_set_member_name(json, "legs");
 	json_builder_begin_array(json);
 
-	for (GList *l = call->monologues.head; l; l = l->next) {
+	for (__auto_type l = call->monologues.head; l; l = l->next) {
 		struct call_monologue *ml = l->data;
 
 		json_builder_begin_object(json);
@@ -467,11 +467,10 @@ static void mqtt_full_call(struct call *call, JsonBuilder *json) {
 
 
 static void mqtt_global_stats(JsonBuilder *json) {
-	AUTO_CLEANUP_INIT(GQueue *metrics, statistics_free_metrics,
-			statistics_gather_metrics(&interface_rate_stats));
+	g_autoptr(stats_metric_q) metrics = statistics_gather_metrics(&interface_rate_stats);
 
-	for (GList *l = metrics->head; l; l = l->next) {
-		struct stats_metric *m = l->data;
+	for (__auto_type l = metrics->head; l; l = l->next) {
+		stats_metric *m = l->data;
 		if (!m->label)
 			continue;
 
@@ -526,7 +525,7 @@ INLINE void __mqtt_timer_outro(JsonBuilder *json) {
 	json_builder_end_object(json);
 	mqtt_publish(glib_json_print(json));
 }
-void mqtt_timer_run_media(struct call *call, struct call_media *media) {
+void mqtt_timer_run_media(call_t *call, struct call_media *media) {
 	JsonBuilder *json = __mqtt_timer_intro();
 
 	rwlock_lock_r(&call->master_lock);
@@ -541,7 +540,7 @@ void mqtt_timer_run_media(struct call *call, struct call_media *media) {
 
 	__mqtt_timer_outro(json);
 }
-void mqtt_timer_run_call(struct call *call) {
+void mqtt_timer_run_call(call_t *call) {
 	JsonBuilder *json = __mqtt_timer_intro();
 
 	mqtt_full_call(call, json);
