@@ -1,10 +1,6 @@
 #include "t38.h"
 
-
-
 #ifdef WITH_TRANSCODING
-
-
 #include <assert.h>
 #include <spandsp/t30.h>
 #include <spandsp/logging.h>
@@ -15,6 +11,7 @@
 #include "str.h"
 #include "media_player.h"
 #include "log_funcs.h"
+#include "sdp.h"
 
 
 
@@ -108,7 +105,7 @@ static int t38_gateway_handler(t38_core_state_t *stat, void *user_data, const ui
 	g_string_append_len(s, (void *) &seq, 2);
 
 	// add primary IFP packet
-	str buf = STR_INIT_LEN((char *) b, len);
+	str buf = STR_INIT_LEN(b, len);
 	__add_udptl(s, &buf);
 
 	// add error correction packets
@@ -220,7 +217,7 @@ static int t38_gateway_handler(t38_core_state_t *stat, void *user_data, const ui
 		ps = tg->t38_media->streams.head->data;
 	if (ps)
 		mutex_lock(&ps->out_lock);
-	struct stream_fd *sfd = NULL;
+	stream_fd *sfd = NULL;
 	if (ps)
 		sfd = ps->selected_sfd;
 	if (sfd && sfd->socket.fd != -1 && ps->endpoint.address.family != NULL) {
@@ -336,6 +333,28 @@ static int span_log_level_map(int level) {
 	return level;
 }
 
+void t38_insert_media_attributes(GString *gs, union sdp_attr_print_arg a, const sdp_ng_flags *flags) {
+	struct t38_gateway *tg = a.cm->t38_gateway;
+	if (!tg)
+		return;
+
+	sdp_append_attr(gs, flags, MT_IMAGE, "T38FaxVersion", "%i", tg->options.version);
+	sdp_append_attr(gs, flags, MT_IMAGE, "T38MaxBitRate", "14400");
+	sdp_append_attr(gs, flags, MT_IMAGE, "T38FaxRateManagement", "%s",
+				tg->options.local_tcf ? "localTFC" : "transferredTCF");
+	sdp_append_attr(gs, flags, MT_IMAGE, "T38FaxMaxBuffer", "1800");
+	sdp_append_attr(gs, flags, MT_IMAGE, "T38FaxMaxDatagram", "512");
+
+	if (tg->options.max_ec_entries == 0)
+		sdp_append_attr(gs, flags, MT_IMAGE, "T38FaxUdpEC", "t38UDPNoEC");
+	else if (tg->options.fec_span > 1)
+		sdp_append_attr(gs, flags, MT_IMAGE, "T38FaxUdpEC", "t38UDPFEC");
+	else
+		sdp_append_attr(gs, flags, MT_IMAGE, "T38FaxUdpEC", "t38UDPRedundancy");
+	// XXX more options possible here
+}
+
+
 // call is locked in W
 int t38_gateway_pair(struct call_media *t38_media, struct call_media *pcm_media,
 		const struct t38_options *options)
@@ -388,9 +407,7 @@ int t38_gateway_pair(struct call_media *t38_media, struct call_media *pcm_media,
 	if (!(tg->gw = t38_gateway_init(NULL, t38_gateway_handler, tg)))
 		goto err;
 
-	err = "Failed to create media player";
-	if (!(tg->pcm_player = media_player_new(pcm_media->monologue)))
-		goto err;
+	media_player_new(&tg->pcm_player, pcm_media->monologue);
 	// even though we call media_player_set_media() here, we need to call it again in
 	// t38_gateway_start because our sink might not have any streams added here yet,
 	// leaving the media_player setup incomplete
@@ -427,22 +444,7 @@ int t38_gateway_pair(struct call_media *t38_media, struct call_media *pcm_media,
 	pcm_media->t38_gateway = obj_get(tg);
 
 	// add SDP options for T38
-	g_queue_clear_full(&t38_media->sdp_attributes, free);
-
-	g_queue_push_tail(&t38_media->sdp_attributes, str_sprintf("T38FaxVersion:%i", tg->options.version));
-	g_queue_push_tail(&t38_media->sdp_attributes, str_sprintf("T38MaxBitRate:14400"));
-	g_queue_push_tail(&t38_media->sdp_attributes, str_sprintf("T38FaxRateManagement:%s",
-				tg->options.local_tcf ? "localTFC" : "transferredTCF"));
-	g_queue_push_tail(&t38_media->sdp_attributes, str_sprintf("T38FaxMaxBuffer:1800"));
-	g_queue_push_tail(&t38_media->sdp_attributes, str_sprintf("T38FaxMaxDatagram:512"));
-
-	if (tg->options.max_ec_entries == 0)
-		g_queue_push_tail(&t38_media->sdp_attributes, str_sprintf("T38FaxUdpEC:t38UDPNoEC"));
-	else if (tg->options.fec_span > 1)
-		g_queue_push_tail(&t38_media->sdp_attributes, str_sprintf("T38FaxUdpEC:t38UDPFEC"));
-	else
-		g_queue_push_tail(&t38_media->sdp_attributes, str_sprintf("T38FaxUdpEC:t38UDPRedundancy"));
-	// XXX more options possible here
+	t38_media->sdp_attr_print = t38_insert_media_attributes;
 
 	return 0;
 
@@ -717,7 +719,7 @@ int t38_gateway_input_udptl(struct t38_gateway *tg, const str *buf) {
 							"packet with seq %i from FEC",
 							seq_fec);
 
-					str rec_str = STR_INIT_LEN(rec_s->str, rec_s->len);
+					str rec_str = STR_INIT_GS(rec_s);
 					__fec_save(tg, &rec_str, seq_fec);
 					up = __make_udptl_packet(&rec_str, seq_fec);
 					packet_sequencer_insert(&tg->sequencer, &up->p);
@@ -775,7 +777,7 @@ void t38_gateway_stop(struct t38_gateway *tg) {
 	if (tg->pcm_player)
 		media_player_stop(tg->pcm_player);
 	if (tg->t38_media)
-		g_queue_clear_full(&tg->t38_media->sdp_attributes, free);
+		tg->t38_media->sdp_attr_print = sdp_insert_media_attributes;
 }
 
 
