@@ -1,12 +1,9 @@
 #ifndef __CALL_H__
 #define __CALL_H__
 
-
-
 /* XXX split everything into call_signalling.[ch] and call_packets.[ch] or w/e */
 
 #include <glib-object.h>
-
 #include <sys/types.h>
 #include <glib.h>
 #include <time.h>
@@ -27,6 +24,8 @@
 #include "statistics.h"
 #include "codeclib.h"
 #include "t38.h"
+#include "types.h"
+
 #include "xt_RTPENGINE.h"
 
 #define UNDEFINED ((unsigned int) -1)
@@ -53,16 +52,66 @@ enum stream_address_format {
 	SAF_NG,
 	SAF_ICE,
 };
+
+enum message_type {
+	SIP_OTHER = 0,
+	SIP_REQUEST,
+	SIP_REPLY,
+};
+
 enum call_opmode {
 	OP_OFFER = 0,
 	OP_ANSWER = 1,
 	OP_REQUEST,
 	OP_REQ_ANSWER,
 	OP_PUBLISH,
+	OP_DELETE,
+	OP_QUERY,
+	OP_LIST,
+	OP_PING,
+	OP_STATISTICS,
+	OP_PLAY_DTMF,
+	OP_BLOCK_DTMF,
+	OP_UNBLOCK_DTMF,
+	OP_BLOCK_MEDIA,
+	OP_UNBLOCK_MEDIA,
+	OP_SILENCE_MEDIA,
+	OP_UNSILENCE_MEDIA,
+	OP_BLOCK_SILENCE_MEDIA,
+	OP_UNBLOCK_SILENCE_MEDIA,
 	OP_PLAY_MEDIA,
+	OP_STOP_MEDIA,
+	OP_START_FORWARDING,
+	OP_STOP_FORWARDING,
+	OP_SUBSCRIBER_REQ,
+	OP_SUBSCRIBER_ANS,
+	OP_UNSUBSCRIBE,
+	OP_START_RECORDING,
+	OP_STOP_RECORDING,
+	OP_PAUSE_RECORDING,
 	OP_OTHER,
 };
 
+#define IS_OP_OTHER(opmode)                                                              \
+		 ((opmode == OP_DELETE || opmode == OP_QUERY)                                    \
+		 || (opmode == OP_LIST || opmode == OP_PING)                                     \
+		 || (opmode == OP_STATISTICS || opmode == OP_PLAY_DTMF)                          \
+		 || (opmode == OP_BLOCK_DTMF || opmode == OP_UNBLOCK_DTMF)                       \
+		 || (opmode == OP_BLOCK_MEDIA || opmode == OP_UNBLOCK_MEDIA)                     \
+		 || (opmode == OP_SILENCE_MEDIA || opmode == OP_UNSILENCE_MEDIA)                 \
+		 || (opmode == OP_BLOCK_SILENCE_MEDIA || opmode == OP_UNBLOCK_SILENCE_MEDIA)     \
+		 || (opmode == OP_PLAY_MEDIA || opmode == OP_STOP_MEDIA)                         \
+		 || (opmode == OP_START_FORWARDING || opmode == OP_STOP_FORWARDING)              \
+		 || (opmode == OP_SUBSCRIBER_REQ || opmode == OP_SUBSCRIBER_ANS)                 \
+		 || (opmode == OP_UNSUBSCRIBE || opmode == OP_START_RECORDING)                   \
+		 || (opmode == OP_STOP_RECORDING || opmode == OP_PAUSE_RECORDING)                \
+		 || (opmode == OP_OTHER))
+
+#define IS_OP_DIRECTIONAL(opmode)                                                        \
+		 ((opmode == OP_BLOCK_DTMF || opmode == OP_BLOCK_MEDIA)                          \
+		 || (opmode == OP_UNBLOCK_DTMF || opmode == OP_UNBLOCK_MEDIA)                    \
+		 || (opmode == OP_START_FORWARDING || opmode == OP_STOP_FORWARDING))
+
 enum call_media_counted {
 	CMC_INCREMENT = 0,
 	CMC_DECREMENT,
@@ -153,13 +202,6 @@ enum {
 #define PS_FLAG_PIERCE_NAT			0x08000000
 #define PS_FLAG_NAT_WAIT			0x10000000
 
-// packet_stream stats_flags
-#define PS_STATS_USERSPACE			0x00000001
-#define PS_STATS_KERNEL				0x00000002
-#define PS_STATS_USERSPACE_COUNTED		0x00000004
-#define PS_STATS_KERNEL_COUNTED			0x00000008
-#define PS_STATS_MIXED_COUNTED			0x00000010
-
 /* struct call_media */
 #define MEDIA_FLAG_INITIALIZED			0x00010000
 #define MEDIA_FLAG_ASYMMETRIC			SHARED_FLAG_ASYMMETRIC
@@ -189,6 +231,8 @@ enum {
 #define MEDIA_FLAG_AUDIO_PLAYER			0x80000000
 #define MEDIA_FLAG_LEGACY_OSRTP			SHARED_FLAG_LEGACY_OSRTP
 #define MEDIA_FLAG_LEGACY_OSRTP_REV		SHARED_FLAG_LEGACY_OSRTP_REV
+#define MEDIA_FLAG_TRANSCODING			0x100000000LL
+#define MEDIA_FLAG_BLOCK_EGRESS			0x200000000LL
 
 /* struct call_monologue */
 #define ML_FLAG_REC_FORWARDING			0x00010000
@@ -196,12 +240,12 @@ enum {
 #define ML_FLAG_DTMF_INJECTION_ACTIVE		0x00040000
 #define ML_FLAG_DETECT_DTMF			0x00080000
 #define ML_FLAG_NO_RECORDING			0x00100000
-#define ML_FLAG_TRANSCODING			0x00200000
+/* unused					0x00200000 */
 #define ML_FLAG_BLOCK_SHORT			0x00400000
 #define ML_FLAG_BLOCK_MEDIA			0x00800000
 #define ML_FLAG_SILENCE_MEDIA			0x01000000
 
-/* struct call */
+/* call_t */
 #define CALL_FLAG_IPV4_OFFER			0x00010000
 #define CALL_FLAG_IPV6_OFFER			0x00020000
 #define CALL_FLAG_IPV4_ANSWER			0x00040000
@@ -216,6 +260,7 @@ enum {
 #define CALL_FLAG_DEBUG				0x08000000
 #define CALL_FLAG_BLOCK_MEDIA			0x10000000
 #define CALL_FLAG_SILENCE_MEDIA			0x20000000
+#define CALL_FLAG_NO_REC_DB			0x40000000
 
 /* access macros */
 #define SP_ISSET(p, f)		bf_isset(&(p)->sp_flags, SP_FLAG_ ## f)
@@ -266,20 +311,18 @@ enum block_dtmf_mode {
 #include "bencode.h"
 #include "crypto.h"
 #include "dtls.h"
+#include "dtmf.h"
 
 
 struct control_stream;
-struct call;
 struct redis;
 struct crypto_suite;
 struct rtpengine_srtp;
-struct sdp_ng_flags;
 struct local_interface;
 struct call_monologue;
 struct ice_agent;
 struct ssrc_hash;
 struct codec_handler;
-struct rtp_payload_type;
 struct media_player;
 struct send_timer;
 struct transport_protocol;
@@ -289,7 +332,7 @@ struct rtcp_timer;
 struct mqtt_timer;
 struct janus_session;
 struct audio_player;
-
+struct media_subscription;
 
 typedef bencode_buffer_t call_buffer_t;
 #define call_buffer_alloc bencode_buffer_alloc
@@ -299,17 +342,28 @@ typedef bencode_buffer_t call_buffer_t;
 
 
 
+TYPED_GHASHTABLE(codecs_ht, void, rtp_payload_type, g_direct_hash, g_direct_equal, NULL, NULL)
+TYPED_GHASHTABLE(codec_names_ht, str, GQueue, str_case_hash, str_case_equal, str_free, g_queue_free)
+TYPED_GHASHTABLE_LOOKUP_INSERT(codec_names_ht, str_free, g_queue_new)
+TYPED_GQUEUE(subscription, struct media_subscription)
+TYPED_DIRECT_FUNCS(media_direct_hash, media_direct_eq, struct call_media)
+TYPED_GHASHTABLE(subscription_ht, struct call_media, subscription_list, media_direct_hash, media_direct_eq,
+		NULL, NULL)
+
+
 struct codec_store {
-	GHashTable		*codecs; // int payload type -> struct rtp_payload_type
-	GHashTable		*codec_names; // codec name -> GQueue of int payload types; storage container
-	GQueue			codec_prefs; // preference by order in SDP; storage container
-	GList			*supp_link; // tracks location for codec_store_add_end
+	codecs_ht		codecs; // int payload type -> rtp_payload_type
+	codec_names_ht		codec_names; // codec name -> GQueue of int payload types; storage container
+	rtp_pt_q		codec_prefs; // preference by order in SDP; storage container
+	rtp_pt_list		*supp_link; // tracks location for codec_store_add_end
 	struct codec_tracker	*tracker;
 	struct call_media	*media;
 	unsigned int		strip_all:1, // set by codec_store_strip
 				strip_full:1; // set by codec_store_strip
 };
 
+TYPED_GQUEUE(endpoint_map, struct endpoint_map)
+
 struct stream_params {
 	unsigned int		index; /* starting with 1 */
 	str			type;
@@ -321,19 +375,22 @@ struct stream_params {
 	str			protocol_str;
 	const struct transport_protocol *protocol;
 	str			format_str;
-	GQueue			sdes_params; // slice-alloc'd
+	sdes_q			sdes_params; // slice-alloc'd
+	sdp_attr_q		attributes;	/* just some other attributes */
 	str			direction[2];
 	sockfamily_t		*desired_family;
 	struct dtls_fingerprint fingerprint;
-	unsigned int		sp_flags;
+	atomic64		sp_flags;
 	struct codec_store	codecs;
-	GQueue			ice_candidates; /* slice-alloc'd */
+	candidate_q		ice_candidates; /* slice-alloc'd */
 	str			ice_ufrag;
 	str			ice_pwd;
 	int			ptime;
 	str			media_id;
 	struct t38_options	t38_options;
 	str			tls_id;
+	int			media_sdp_id;
+	int			media_session_as, media_session_rr, media_session_rs;
 };
 
 struct endpoint_map {
@@ -341,7 +398,7 @@ struct endpoint_map {
 	struct endpoint		endpoint;
 	unsigned int		num_ports;
 	const struct logical_intf *logical_intf;
-	GQueue			intf_sfds; /* list of struct intf_list - contains stream_fd list */
+	sfd_intf_list_q		intf_sfds; /* list of struct sfd_intf_list - contains stream_fd list */
 	unsigned int		wildcard:1;
 };
 
@@ -375,19 +432,19 @@ struct packet_stream {
 				out_lock;
 
 	struct call_media	*media;		/* RO */
-	struct call		*call;		/* RO */
+	call_t		*call;		/* RO */
 	unsigned int		component;	/* RO, starts with 1 */
 	unsigned int		unique_id;	/* RO */
 	struct recording_stream recording;	/* LOCK: call->master_lock */
 
-	GQueue			sfds;		/* LOCK: call->master_lock */
-	struct stream_fd *	selected_sfd;
+	stream_fd_q		sfds;		/* LOCK: call->master_lock */
+	stream_fd *	selected_sfd;
 	endpoint_t		last_local_endpoint;
 	struct dtls_connection	ice_dtls;	/* LOCK: in_lock */
-	GQueue			rtp_sinks;	/* LOCK: call->master_lock, in_lock for streamhandler */
-	GQueue			rtcp_sinks;	/* LOCK: call->master_lock, in_lock for streamhandler */
+	sink_handler_q		rtp_sinks;	/* LOCK: call->master_lock, in_lock for streamhandler */
+	sink_handler_q		rtcp_sinks;	/* LOCK: call->master_lock, in_lock for streamhandler */
 	struct packet_stream	*rtcp_sibling;	/* LOCK: call->master_lock */
-	GQueue			rtp_mirrors;	/* LOCK: call->master_lock, in_lock for streamhandler */
+	sink_handler_q		rtp_mirrors;	/* LOCK: call->master_lock, in_lock for streamhandler */
 	struct endpoint		endpoint;	/* LOCK: out_lock */
 	struct endpoint		detected_endpoints[4];		/* LOCK: out_lock */
 	struct timeval		ep_detect_signal;		/* LOCK: out_lock */
@@ -402,15 +459,11 @@ struct packet_stream {
 	struct jitter_buffer	*jb;					/* RO */
 	time_t kernel_time;
 
-	struct stream_stats	stats_in;
-	struct stream_stats	stats_out;
-	struct stream_stats	kernel_stats_in;
-	struct stream_stats	kernel_stats_out;
-	unsigned char		in_tos_tclass;
-	atomic64		last_packet;
+	struct stream_stats	*stats_in;
+	struct stream_stats	*stats_out;
+	atomic64		last_packet;				// userspace only
 	GHashTable		*rtp_stats;				/* LOCK: call->master_lock */
 	struct rtp_stats	*rtp_stats_cache;
-	unsigned int		stats_flags;
 	enum endpoint_learning		el_flags;
 
 #if RTP_LOOP_PROTECT
@@ -423,9 +476,15 @@ struct packet_stream {
 	X509			*dtls_cert;				/* LOCK: in_lock */
 
 	/* in_lock must be held for SETTING these: */
-	volatile unsigned int	ps_flags;
+	atomic64		ps_flags;
 };
 
+INLINE uint64_t packet_stream_last_packet(const struct packet_stream *ps) {
+	uint64_t lp1 = atomic64_get_na(&ps->last_packet);
+	uint64_t lp2 = atomic64_get_na(&ps->stats_in->last_packet);
+	return MAX(lp1, lp2);
+}
+
 /**
  * Protected by call->master_lock, except the RO elements.
  * 
@@ -434,7 +493,7 @@ struct packet_stream {
  */
 struct call_media {
 	struct call_monologue	*monologue;			/* RO */
-	struct call		*call;				/* RO */
+	call_t		*call;				/* RO */
 
 	unsigned int		index;				/* RO */
 	unsigned int		unique_id;			/* RO */
@@ -450,19 +509,20 @@ struct call_media {
 
 	str			media_id;
 	str			label;
-	GQueue			sdes_in, sdes_out;
+	sdes_q			sdes_in, sdes_out;
 	struct dtls_fingerprint fingerprint;			/* as received */
 	const struct dtls_hash_func *fp_hash_func;		/* outgoing */
 	str			tls_id;
 
-	GQueue			streams;			/* normally RTP + RTCP */
-	GQueue			endpoint_maps;
+	packet_stream_q		streams;			/* normally RTP + RTCP */
+	endpoint_map_q		endpoint_maps;
 
 	struct codec_store	codecs;
-	GQueue			sdp_attributes;			/* str_sprintf() */
-	GHashTable		*codec_handlers;		/* int payload type -> struct codec_handler
+	sdp_attr_q		sdp_attributes;			/* sdp_attr_new() */
+	sdp_attr_print_f	*sdp_attr_print;
+	codec_handlers_ht	codec_handlers;			/* int payload type -> struct codec_handler
 														XXX combine this with 'codecs' hash table? */
-	GQueue			codec_handlers_store;		/* storage for struct codec_handler */
+	codec_handlers_q	codec_handlers_store;		/* storage for struct codec_handler */
 	struct codec_handler	*codec_handler_cache;
 	struct rtcp_handler	*rtcp_handler;
 	struct rtcp_timer	*rtcp_timer;			/* master lock for scheduling purposes */
@@ -474,12 +534,22 @@ struct call_media {
 
 	unsigned int		buffer_delay;
 
+	/* media subsriptions handling */
+	subscription_ht		media_subscriptions_ht;		/* for quick lookup of our subsriptions */
+	subscription_ht		media_subscribers_ht;		/* for quick lookup of medias subscribed to us */
+	subscription_q		media_subscribers;		/* who is subscribed to this media (sinks) */
+	subscription_q		media_subscriptions;		/* who am I subscribed to (sources) */
+
 	mutex_t			dtmf_lock;
 	unsigned long		dtmf_ts;			/* TS of last processed end event */
 	unsigned int		dtmf_count;
 	// lists are append-only
-	GQueue			dtmf_recv;
-	GQueue			dtmf_send;
+	dtmf_event_q		dtmf_recv;
+	dtmf_event_q		dtmf_send;
+	int					media_sdp_id;
+
+	/* bandwidth */
+	int bandwidth_as, bandwidth_rr, bandwidth_rs;
 
 #ifdef WITH_TRANSCODING
 	encoder_callback_t	encoder_callback;
@@ -487,26 +557,18 @@ struct call_media {
 
 	int			ptime;				/* either from SDP or overridden */
 
-	volatile unsigned int	media_flags;
+	atomic64		media_flags;
 };
 
-/** 
- * Link between subscribers and subscriptions.
- * 
- * Contain flags and attributes, which can be used
- * to mark a subscription (for example, as an egress subscription).
- * 
- * During signalling events, the list of subscriptions for each call_monologue
- * is used to create the list of rtp_sink and rtcp_sink given in each packet_stream.
- * 
- * Each entry in these lists is a sink_handler object, which again contains flags and attributes.
- * Flags from a call_subscription are copied into the sink_handler.
- */
-struct call_subscription {
-	struct call_monologue	*monologue;
-	GList			*link; // link into the corresponding opposite list
-	unsigned int		media_offset; // 0 if media indexes match up
-	struct sink_attrs	attrs;
+TYPED_GPTRARRAY(medias_arr, struct call_media)
+TYPED_GQUEUE(medias, struct call_media)
+
+
+struct media_subscription {
+	struct call_media	* media;	/* media itself */
+	struct call_monologue	* monologue;	/* whom media belongs to */
+	struct sink_attrs	attrs;		/* attributes to passed to a sink */
+	subscription_list	* link;		/* TODO: is this still really needed? */
 };
 
 /**
@@ -520,7 +582,7 @@ struct call_subscription {
  * A regular A/B call has two call_monologue objects with each subscribed to the other.
  */
 struct call_monologue {
-	struct call		*call;			/* RO */
+	call_t		*call;			/* RO */
 	unsigned int		unique_id;		/* RO */
 
 	str			tag;
@@ -535,48 +597,59 @@ struct call_monologue {
 	sockfamily_t		*desired_family;
 	const struct logical_intf *logical_intf;
 	GHashTable 		*associated_tags;
-	GQueue			subscriptions;		/* who am I subscribed to (sources) */
-	GHashTable		*subscriptions_ht;	/* for quick lookup */
-	GQueue			subscribers;		/* who is subscribed to me (sinks) */
 	GHashTable		*subscribers_ht;	/* for quick lookup */
-	GPtrArray		*medias;
+	medias_arr		*medias;
 	GHashTable		*media_ids;
 	struct media_player	*player;
-	unsigned long long	sdp_session_id;
-	unsigned long long	sdp_version;
+	struct media_player	*rec_player;
+	int			sdp_session_rr, sdp_session_rs;
 	str			last_in_sdp;
-	GQueue			last_in_sdp_parsed;
-	GQueue			last_in_sdp_streams;
+	sdp_sessions_q		last_in_sdp_parsed;	/* last parsed `sdp_session` */
+	sdp_streams_q		last_in_sdp_streams;	/* last parsed `stream_params` */
 	GString			*last_out_sdp;
-	char			*sdp_username;
+
+	sdp_origin * session_sdp_orig;	/* actual origin belonging to this monologue */
+	sdp_origin * session_last_sdp_orig;	/* previously used origin by other other side */
+
 	char			*sdp_session_name;
+	char			*sdp_session_timing;
 	struct ssrc_hash	*ssrc_hash;
 	str			metadata;
 	struct janus_session	*janus_session;
 
-	// DTMF blocking/replacement stuff:
-	enum block_dtmf_mode	block_dtmf;
-	GArray			*tone_freqs;
-	unsigned int		tone_vol;
-	char			dtmf_digit;
-	str			dtmf_trigger;
-	unsigned int		dtmf_trigger_match;
-	enum block_dtmf_mode	block_dtmf_trigger;
-	str			dtmf_trigger_end;
-	int			dtmf_trigger_digits;
-	enum block_dtmf_mode	block_dtmf_trigger_end;
-	unsigned int		block_dtmf_trigger_end_ms;
+	// DTMF triggers, MUST be set via dtmf_trigger_set() only
+	struct dtmf_trigger_state dtmf_trigger_state[__NUM_DTMF_TRIGGERS];
+	uint8_t			dtmf_trigger_index[__NUM_DTMF_TRIGGERS];
+	unsigned int		num_dtmf_triggers;
 	unsigned int		dtmf_delay;
 
-	volatile unsigned int	ml_flags;
+	// DTMF blocking/replacement stuff:
+	enum block_dtmf_mode	block_dtmf; // current block mode
+	GArray			*tone_freqs;
+	unsigned int		tone_vol;
+	char			dtmf_digit; // replacement digit
+	enum block_dtmf_mode	block_dtmf_trigger; // to enable when trigger detected
+	int			dtmf_trigger_digits; // unblock after this many digits
+	enum block_dtmf_mode	block_dtmf_trigger_end; // to enable when trigger detected
+	unsigned int		block_dtmf_trigger_end_ms; // unblock after this many ms
+
+	/* carry `sdp_session` attributes into resulting call monologue SDP */
+	sdp_attr_q		sdp_attributes;
+	sdp_attr_print_f	*sdp_attr_print;
+
+	atomic64		ml_flags;
 };
 
+TYPED_GQUEUE(monologues, struct call_monologue)
+G_DEFINE_AUTO_CLEANUP_CLEAR_FUNC(monologues_q, monologues_q_clear)
+TYPED_GHASHTABLE(tags_ht, str, struct call_monologue, str_hash, str_equal, NULL, NULL)
+
 struct call_iterator_list {
-	GList *first;
+	call_list *first;
 	mutex_t lock; // protects .first and every entry's .data
 };
 struct call_iterator_entry {
-	GList link; // .data is protected by the list's main lock
+	call_list link; // .data is protected by the list's main lock
 	mutex_t next_lock; // held while the link is in use, protects link.data and link.next
 	mutex_t prev_lock; // held while the link is in use, protects link.prev
 };
@@ -586,11 +659,11 @@ struct call_iterator_entry {
 		int __which = (which); \
 		mutex_lock(&rtpe_call_iterators[__which].lock); \
 		\
-		GList *__l = rtpe_call_iterators[__which].first; \
+		__auto_type __l = rtpe_call_iterators[__which].first; \
 		bool __has_lock = true; \
-		struct call *next_ ## varname = NULL; \
+		call_t *next_ ## varname = NULL; \
 		while (__l) { \
-			struct call *varname = NULL; \
+			call_t *varname = NULL; \
 			if (next_ ## varname) \
 				varname = next_ ## varname; \
 			else { \
@@ -603,7 +676,7 @@ struct call_iterator_entry {
 			__has_lock = false
 
 #define ITERATE_CALL_LIST_NEXT_END(varname) \
-			GList *__next = varname->iterator[__which].link.next; \
+			__auto_type __next = varname->iterator[__which].link.next; \
 			if (__next) { \
 				next_ ## varname = __next->data; \
 				obj_hold(next_ ## varname); \
@@ -619,13 +692,16 @@ struct call_iterator_entry {
 			mutex_unlock(&rtpe_call_iterators[__which].lock); \
 	} while (0)
 
+
+TYPED_GHASHTABLE(labels_ht, str, struct call_monologue, str_hash, str_equal, NULL, NULL)
+
 /**
- * struct call is the main parent structure of all call-related objects.
+ * call_t is the main parent structure of all call-related objects.
  * 
  * The logical object hierarchy under the 'struct call':
  * call > call_monologue > call_media > packet_stream > stream_fd
  * 
- * struct call usually has multiple call_monologue objects.
+ * call_t usually has multiple call_monologue objects.
  * Meanwhile each sub-object of call, as a parent of own sub-objects,
  * can also contain multiple child objects.
  * 
@@ -653,6 +729,9 @@ struct call {
 	mutex_t			buffer_lock;
 	call_buffer_t		buffer;
 
+	// use a single poller for all sockets within the call
+	struct poller		*poller;
+
 	/* master_lock protects the entire call and all the contained objects.
 	 * 
 	 * All the fields and any nested sub-object must:
@@ -665,14 +744,14 @@ struct call {
 	rwlock_t		master_lock;
 
 	/* everything below is protected by the master_lock */
-	GQueue			monologues;	/* call_monologue */
-	GQueue			medias;		/* call_media */
-	GHashTable		*tags;
-	GHashTable		*viabranches;
-	GHashTable		*labels;
-	GQueue			streams;
-	GQueue			stream_fds;	/* stream_fd */
-	GQueue			endpoint_maps;
+	monologues_q		monologues;	/* call_monologue */
+	medias_q		medias;		/* call_media */
+	tags_ht			tags;
+	tags_ht			viabranches;
+	labels_ht		labels;
+	packet_stream_q		streams;
+	stream_fd_q		stream_fds;	/* stream_fd */
+	endpoint_map_q		endpoint_maps;
 	struct dtls_cert	*dtls_cert;	/* for outgoing */
 	struct mqtt_timer	*mqtt_timer;
 
@@ -689,15 +768,21 @@ struct call {
 	endpoint_t		dtmf_log_dest;
 
 	unsigned int		redis_hosted_db;
+	atomic64		last_redis_update;
 
 	struct recording 	*recording;
 	str			metadata;
+	str			recording_meta_prefix;
+	str			recording_file;
+	str			recording_random_tag;
+	str			recording_path;
+	str			recording_pattern;
 
 	struct call_iterator_entry iterator[NUM_CALL_ITERATORS];
 	int			cpu_affinity;
 	enum block_dtmf_mode	block_dtmf;
 
-	unsigned int		call_flags;
+	atomic64		call_flags;
 };
 
 
@@ -707,8 +792,10 @@ struct call {
  * which uses call-IDs as keys and call objects as values,
  * while holding a reference to each contained call.
  */
+TYPED_GHASHTABLE(rtpe_calls_ht, str, struct call, str_hash, str_equal, NULL, NULL)
+
 extern rwlock_t rtpe_callhash_lock;
-extern GHashTable *rtpe_callhash;
+extern rtpe_calls_ht rtpe_callhash;
 extern struct call_iterator_list rtpe_call_iterators[NUM_CALL_ITERATORS];
 
 
@@ -716,52 +803,58 @@ extern struct call_iterator_list rtpe_call_iterators[NUM_CALL_ITERATORS];
 int call_init(void);
 void call_free(void);
 
-struct call_monologue *__monologue_create(struct call *call);
+struct call_monologue *__monologue_create(call_t *call);
 void __monologue_free(struct call_monologue *m);
 void __monologue_tag(struct call_monologue *ml, const str *tag);
 void __monologue_viabranch(struct call_monologue *ml, const str *viabranch);
-struct packet_stream *__packet_stream_new(struct call *call);
-void __add_subscription(struct call_monologue *ml, struct call_monologue *other,
-		unsigned int media_offset, const struct sink_attrs *);
-struct call_subscription *call_get_call_subscription(GHashTable *ht, struct call_monologue *ml);
-void free_sink_handler(void *);
-void __add_sink_handler(GQueue *, struct packet_stream *, const struct sink_attrs *);
+struct packet_stream *__packet_stream_new(call_t *call);
+void __add_media_subscription(struct call_media * which, struct call_media * to,
+		const struct sink_attrs *attrs);
+struct media_subscription *call_get_top_media_subscription(struct call_monologue *ml);
+struct media_subscription *call_get_media_subscription(subscription_ht ht, struct call_media * cm);
+struct call_monologue * ml_medias_subscribed_to_single_ml(struct call_monologue *ml);
 
-void call_subscription_free(void *);
-void call_subscriptions_clear(GQueue *q);
+void free_sink_handler(struct sink_handler *);
+void __add_sink_handler(sink_handler_q *, struct packet_stream *, const struct sink_attrs *);
 
+void media_subscription_free(struct media_subscription *);
+void media_subscriptions_clear(subscription_q *q);
+G_DEFINE_AUTO_CLEANUP_CLEAR_FUNC(subscription_q, media_subscriptions_clear)
 
-struct call *call_get_or_create(const str *callid, bool exclusive);
-struct call *call_get_opmode(const str *callid, enum call_opmode opmode);
-void call_make_own_foreign(struct call *c, bool foreign);
-int call_get_mono_dialogue(struct call_subscription *dialogue[2], struct call *call, const str *fromtag,
+call_t *call_get_or_create(const str *callid, bool exclusive);
+call_t *call_get_opmode(const str *callid, enum call_opmode opmode);
+void call_make_own_foreign(call_t *c, bool foreign);
+int call_get_mono_dialogue(struct call_monologue *monologues[2], call_t *call,
+		const str *fromtag,
 		const str *totag,
 		const str *viabranch);
-struct call_monologue *call_get_monologue(struct call *call, const str *fromtag);
-struct call_monologue *call_get_or_create_monologue(struct call *call, const str *fromtag);
-struct call *call_get(const str *callid);
-int monologue_offer_answer(struct call_subscription *dialogue[2], GQueue *streams, struct sdp_ng_flags *flags);
-__attribute__((nonnull(1, 2, 3, 5)))
+struct call_monologue *call_get_monologue(call_t *call, const str *fromtag);
+struct call_monologue *call_get_or_create_monologue(call_t *call, const str *fromtag);
+call_t *call_get(const str *callid);
+__attribute__((nonnull(2, 3)))
+int monologue_offer_answer(struct call_monologue *monologues[2], sdp_streams_q *streams, sdp_ng_flags *flags);
+__attribute__((nonnull(1, 2, 3, 4)))
 void codecs_offer_answer(struct call_media *media, struct call_media *other_media,
-		struct stream_params *sp, struct sdp_ng_flags *flags, struct call_subscription *dialogue[2]);
-int monologue_publish(struct call_monologue *ml, GQueue *streams, struct sdp_ng_flags *flags);
-int monologue_subscribe_request(const GQueue *srcs, struct call_monologue *dst, struct sdp_ng_flags *);
-int monologue_subscribe_answer(struct call_monologue *dst, struct sdp_ng_flags *,
-		GQueue *);
-int monologue_unsubscribe(struct call_monologue *dst, struct sdp_ng_flags *);
+		struct stream_params *sp, sdp_ng_flags *flags);
+int monologue_publish(struct call_monologue *ml, sdp_streams_q *streams, sdp_ng_flags *flags);
+int monologue_subscribe_request(const subscription_q *srms, struct call_monologue *dst, sdp_ng_flags *flags);
+int monologue_subscribe_answer(struct call_monologue *dst, sdp_ng_flags *flags,
+		sdp_streams_q *streams);
+int monologue_unsubscribe(struct call_monologue *dst, sdp_ng_flags *);
 void monologue_destroy(struct call_monologue *ml);
 int call_delete_branch_by_id(const str *callid, const str *branch,
 	const str *fromtag, const str *totag, bencode_item_t *output, int delete_delay);
-int call_delete_branch(struct call *, const str *branch,
+int call_delete_branch(call_t *, const str *branch,
 	const str *fromtag, const str *totag, bencode_item_t *output, int delete_delay);
-void call_destroy(struct call *);
-struct call_media *call_media_new(struct call *call);
+void call_destroy(call_t *);
+struct call_media *call_media_new(call_t *call);
 void call_media_free(struct call_media **mdp);
 enum call_stream_state call_stream_state_machine(struct packet_stream *);
 void call_media_state_machine(struct call_media *m);
-void call_media_unkernelize(struct call_media *media, const char *);
-void dialogue_unkernelize(struct call_monologue *ml, const char *);
-void __monologue_unkernelize(struct call_monologue *monologue, const char *);
+void call_media_unkernelize(struct call_media *media, const char *reason);
+void dialogue_unconfirm(struct call_monologue *ml, const char *);
+void __monologue_unconfirm(struct call_monologue *monologue, const char *);
+void __media_unconfirm(struct call_media *media, const char *);
 void update_init_subscribers(struct call_monologue *ml, enum call_opmode opmode);
 
 int call_stream_address46(char *o, struct packet_stream *ps, enum stream_address_format format,
@@ -774,12 +867,12 @@ void __rtp_stats_update(GHashTable *dst, struct codec_store *);
 int __init_stream(struct packet_stream *ps);
 void call_stream_crypto_reset(struct packet_stream *ps);
 
-const struct rtp_payload_type *__rtp_stats_codec(struct call_media *m);
+const rtp_payload_type *__rtp_stats_codec(struct call_media *m);
 
 #include "str.h"
 #include "rtp.h"
 
-INLINE void *call_malloc(struct call *c, size_t l) {
+INLINE void *call_malloc(call_t *c, size_t l) {
 	void *ret;
 	mutex_lock(&c->buffer_lock);
 	ret = call_buffer_alloc(&c->buffer, l);
@@ -787,7 +880,7 @@ INLINE void *call_malloc(struct call *c, size_t l) {
 	return ret;
 }
 
-INLINE char *call_strdup_len(struct call *c, const char *s, unsigned int len) {
+INLINE char *call_strdup_len(call_t *c, const char *s, unsigned int len) {
 	char *r;
 	if (!s)
 		return NULL;
@@ -797,12 +890,12 @@ INLINE char *call_strdup_len(struct call *c, const char *s, unsigned int len) {
 	return r;
 }
 
-INLINE char *call_strdup(struct call *c, const char *s) {
+INLINE char *call_strdup(call_t *c, const char *s) {
 	if (!s)
 		return NULL;
 	return call_strdup_len(c, s, strlen(s));
 }
-INLINE str *call_str_cpy_len(struct call *c, str *out, const char *in, int len) {
+INLINE str *call_str_cpy_len(call_t *c, str *out, const char *in, int len) {
 	if (!in) {
 		*out = STR_NULL;
 		return out;
@@ -811,27 +904,27 @@ INLINE str *call_str_cpy_len(struct call *c, str *out, const char *in, int len)
 	out->len = len;
 	return out;
 }
-INLINE str *call_str_cpy(struct call *c, str *out, const str *in) {
+INLINE str *call_str_cpy(call_t *c, str *out, const str *in) {
 	return call_str_cpy_len(c, out, in ? in->s : NULL, in ? in->len : 0);
 }
-INLINE str *call_str_cpy_c(struct call *c, str *out, const char *in) {
+INLINE str *call_str_cpy_c(call_t *c, str *out, const char *in) {
 	return call_str_cpy_len(c, out, in, in ? strlen(in) : 0);
 }
-INLINE str *call_str_dup(struct call *c, const str *in) {
+INLINE str *call_str_dup(call_t *c, const str *in) {
 	str *out;
 	out = call_malloc(c, sizeof(*out));
 	call_str_cpy_len(c, out, in->s, in->len);
 	return out;
 }
-INLINE str *call_str_init_dup(struct call *c, char *s) {
+INLINE str *call_str_init_dup(call_t *c, char *s) {
 	str t;
 	str_init(&t, s);
 	return call_str_dup(c, &t);
 }
-INLINE void __call_unkernelize(struct call *call, const char *reason) {
-	for (GList *l = call->monologues.head; l; l = l->next) {
+INLINE void __call_unkernelize(call_t *call, const char *reason) {
+	for (__auto_type l = call->monologues.head; l; l = l->next) {
 		struct call_monologue *ml = l->data;
-		__monologue_unkernelize(ml, reason);
+		__monologue_unconfirm(ml, reason);
 	}
 }
 INLINE endpoint_t *packet_stream_local_addr(struct packet_stream *ps) {
