@@ -1,11 +1,12 @@
 #ifndef _MAIN_H_
 #define _MAIN_H_
 
+#include <glib.h>
 
 #include "helpers.h"
-#include <glib.h>
 #include "socket.h"
 #include "auxlib.h"
+#include "types.h"
 
 enum xmlrpc_format {
 	XF_SEMS = 0,
@@ -27,9 +28,12 @@ enum endpoint_learning {
 	__EL_LAST
 };
 
+#ifndef MAX_RECV_ITERS
+#define MAX_RECV_ITERS 50
+#endif
+
 struct rtpengine_config {
-	/* everything below protected by config_lock */
-	rwlock_t		config_lock;
+	rwlock_t		keyspaces_lock;
 
 	struct rtpengine_common_config common;
 
@@ -56,7 +60,7 @@ struct rtpengine_config {
 	int			graphite_interval;
 	int			graphite_timeout;
 	int			redis_num_threads;
-	GQueue			interfaces;
+	intf_config_q		interfaces;
 	GQueue			tcp_listen_ep;
 	GQueue			udp_listen_ep;
 	GQueue			ng_listen_ep;
@@ -67,6 +71,9 @@ struct rtpengine_config {
 	endpoint_t		homer_ep;
 	int			homer_protocol;
 	int			homer_id;
+	int			homer_ng_capt_proto;
+	gboolean		homer_rtcp_off;
+	gboolean		homer_ng_on;
 	gboolean		no_fallback;
 	gboolean		reject_invalid_sdp;
 	gboolean		save_interface_ports;
@@ -86,11 +93,16 @@ struct rtpengine_config {
 	gboolean		active_switchover;
 	int			num_threads;
 	int			media_num_threads;
+	int			codec_num_threads;
 	char			*spooldir;
 	char			*rec_method;
 	char			*rec_format;
 	gboolean		rec_egress;
 	char			*iptables_chain;
+	char			*nftables_chain;
+	char			*nftables_base_chain;
+	gboolean		nftables_append;
+	int			nftables_family;
 	int			load_limit;
 	int			cpu_limit;
 	uint64_t		bw_limit;
@@ -139,6 +151,8 @@ struct rtpengine_config {
 	uint32_t		silence_detect_int;
 	str			cn_payload;
 	gboolean		player_cache;
+	int			kernel_player;
+	int			kernel_player_media;
 	int			audio_buffer_length;
 	int			audio_buffer_delay;
 	enum {
@@ -175,13 +189,20 @@ struct rtpengine_config {
 		MOS_LQ,
 	}			mos;
 	gboolean		measure_rtp;
+	int			rtcp_interval;
 	int			cpu_affinity;
 	char			*janus_secret;
+	int			max_recv_iters;
+	str			vsc_start_rec;
+	str			vsc_stop_rec;
+	str			vsc_start_stop_rec;
+	str			vsc_pause_rec;
+	str			vsc_pause_resume_rec;
+	str			vsc_start_pause_resume_rec;
 };
 
 
 struct poller;
-struct poller_map;
 
 /**
  * Main global poller instance.
@@ -189,9 +210,16 @@ struct poller_map;
  *
  *  TODO: convert to struct instead of pointer?
  */
-extern struct poller *rtpe_poller;
-/* Used when the poller-per-thread option is set */
-extern struct poller_map *rtpe_poller_map;
+extern struct poller **rtpe_pollers; // at least one poller, in an array
+extern struct poller *rtpe_control_poller; // poller for control sockets (maybe rtpe_pollers[0])
+extern unsigned int num_media_pollers; // for media sockets, >= 1
+extern unsigned int rtpe_poller_rr_iter; // round-robin assignment of pollers to each thread
+
+
+INLINE struct poller *rtpe_get_poller(void) {
+	// XXX optimise this for num_media_pollers == 1 ?
+	return rtpe_pollers[g_atomic_int_add(&rtpe_poller_rr_iter, 1) % num_media_pollers];
+}
 
 extern struct rtpengine_config rtpe_config;
 extern struct rtpengine_config initial_rtpe_config;
@@ -199,6 +227,7 @@ extern struct rtpengine_config initial_rtpe_config;
 extern GQueue rtpe_control_ng;
 extern GQueue rtpe_control_ng_tcp;
 
+extern struct bufferpool *shm_bufferpool;
 
 
 #endif
