@@ -1,15 +1,13 @@
 #ifndef _MEDIA_PLAYER_H_
 #define _MEDIA_PLAYER_H_
 
+#include <stdbool.h>
 
 #include "auxlib.h"
 #include "timerthread.h"
 #include "str.h"
-#include <stdbool.h>
+#include "types.h"
 
-
-
-struct call;
 struct call_media;
 struct call_monologue;
 struct codec_handler;
@@ -17,7 +15,13 @@ struct ssrc_ctx;
 struct packet_stream;
 struct codec_packet;
 struct media_player;
-struct rtp_payload_type;
+
+
+typedef struct {
+	long long start_pos;
+	int repeat;
+	unsigned int block_egress:1;
+} media_player_opts_t;
 
 
 #ifdef WITH_TRANSCODING
@@ -54,19 +58,21 @@ struct media_player {
 	struct timerthread_obj tt_obj;
 	mutex_t lock;
 	media_player_run_func run_func;
-	struct call *call;
+	call_t *call;
 	struct call_monologue *ml;
 	struct call_media *media;
 	struct packet_stream *sink;
 	const struct streamhandler *crypt_handler;
 
 	struct timeval next_run;
-	unsigned long repeat;
+
+	media_player_opts_t opts;
 
 	struct media_player_coder coder;
 	struct media_player_content_index cache_index;
 	struct media_player_cache_entry *cache_entry;
 	unsigned int cache_read_idx;
+	unsigned int kernel_idx;
 
 	struct ssrc_ctx *ssrc_out;
 	unsigned long seq;
@@ -101,35 +107,40 @@ INLINE struct media_player *media_player_get(struct media_player *mp) {
 
 struct send_timer {
 	struct timerthread_queue ttq;
-	struct call *call; // main reference that keeps this alive
+	call_t *call; // main reference that keeps this alive
 	struct packet_stream *sink;
 };
 
 
-struct media_player *media_player_new(struct call_monologue *);
-int media_player_play_file(struct media_player *, const str *, long long, long long);
-int media_player_play_blob(struct media_player *, const str *, long long, long long);
-int media_player_play_db(struct media_player *, long long, long long, long long);
+#define MPO(...) (media_player_opts_t){__VA_ARGS__}
+
+void media_player_new(struct media_player **, struct call_monologue *);
+bool media_player_play_file(struct media_player *, const str *, media_player_opts_t);
+bool media_player_play_blob(struct media_player *, const str *, media_player_opts_t);
+bool media_player_play_db(struct media_player *, long long, media_player_opts_t);
+bool media_player_init_file(struct media_player *, const str *, media_player_opts_t);
+bool media_player_init_blob(struct media_player *, const str *, media_player_opts_t);
+bool media_player_init_db(struct media_player *, long long, media_player_opts_t);
+bool media_player_start(struct media_player *);
 long long media_player_stop(struct media_player *);
 bool media_player_is_active(struct call_monologue *);
 
-int media_player_setup(struct media_player *mp, const struct rtp_payload_type *src_pt,
-		const struct rtp_payload_type *dst_pt);
+int media_player_setup(struct media_player *mp, const rtp_payload_type *src_pt,
+		const rtp_payload_type *dst_pt);
 void media_player_set_media(struct media_player *mp, struct call_media *media);
-bool media_player_pt_match(const struct media_player *mp, const struct rtp_payload_type *src_pt,
-		const struct rtp_payload_type *dst_pt);
+bool media_player_pt_match(const struct media_player *mp, const rtp_payload_type *src_pt,
+		const rtp_payload_type *dst_pt);
 
 void media_player_add_packet(struct media_player *mp, char *buf, size_t len,
 		long long us_dur, unsigned long long pts);
 
 void media_player_init(void);
 void media_player_free(void);
-void media_player_loop(void *);
+void media_player_launch(void);
 
 struct send_timer *send_timer_new(struct packet_stream *);
 void send_timer_push(struct send_timer *, struct codec_packet *);
-
-void send_timer_loop(void *p);
+void send_timer_launch(void);
 
 
 
