@@ -1,12 +1,10 @@
 #ifndef _RTP_H_
 #define _RTP_H_
 
-
-
-#include "str.h"
 #include <glib.h>
 
-
+#include "str.h"
+#include "types.h"
 
 struct crypto_context;
 struct rtp_header;
@@ -15,11 +13,7 @@ enum ssrc_dir;
 struct ssrc_ctx;
 struct codec_store;
 
-
-
-
-
-const struct rtp_payload_type *rtp_payload_type(unsigned int, struct codec_store *);
+const rtp_payload_type *get_rtp_payload_type(unsigned int, struct codec_store *);
 
 int rtp_avp2savp(str *, struct crypto_context *, struct ssrc_ctx *);
 int rtp_savp2avp(str *, struct crypto_context *, struct ssrc_ctx *);
@@ -29,7 +23,4 @@ int srtp_payloads(str *to_auth, str *to_decrypt, str *auth_tag, str *mki,
 		int auth_len, int mki_len,
 		const str *packet, const str *payload);
 
-
-
-
 #endif
