@@ -24,19 +24,25 @@ struct rtpengine_common_config {
 	char *log_facility;
 	int default_log_level;
 	int log_levels[MAX_LOG_LEVELS];
-	int log_stderr;
-	int split_logs;
-	int no_log_timestamps;
+	gboolean log_stderr;
+	gboolean split_logs;
+	gboolean no_log_timestamps;
 	char *log_name;
 	char *log_mark_prefix;
 	char *log_mark_suffix;
 	char *pidfile;
-	int foreground;
+	gboolean foreground;
 	int thread_stack;
 	int poller_size;
+	gboolean io_uring;
+	int io_uring_buffers;
 	int max_log_line_length;
 	char *evs_lib_path;
 	char *codec_chain_lib_path;
+	int codec_chain_runners;
+	int codec_chain_concurrency;
+	int codec_chain_async;
+	int codec_chain_opus_complexity;
 };
 
 extern struct rtpengine_common_config *rtpe_common_config_ptr;
@@ -78,14 +84,10 @@ int num_cpu_cores(int);
 
 #define UINT64F			"%" G_GUINT64_FORMAT
 
-void free_gbuf(char **);
-void free_gvbuf(char ***);
-
-#define AUTO_CLEANUP(decl, func)		decl __attribute__ ((__cleanup__(func)))
-#define AUTO_CLEANUP_INIT(decl, func, val)	AUTO_CLEANUP(decl, func) = val
-#define AUTO_CLEANUP_NULL(decl, func)		AUTO_CLEANUP_INIT(decl, func, 0)
-#define AUTO_CLEANUP_GBUF(var)			AUTO_CLEANUP_NULL(char *var, free_gbuf)
-#define AUTO_CLEANUP_GVBUF(var)			AUTO_CLEANUP_NULL(char **var, free_gvbuf)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(char, g_free)
+typedef char *char_p;
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(char_p, g_strfreev)
+#define auto_iter(v, l) __typeof__ ( ({ __typeof__ (*l) __t; &__t; }) ) v = (l) /* for gcc <12 */
 
 
 /*** STRING HELPERS ***/
@@ -96,6 +98,13 @@ INLINE void random_string(unsigned char *buf, int len) {
 	(void) ret;
 }
 
+INLINE unsigned int c_str_hash(const char *s) {
+	return g_str_hash(s);
+}
+INLINE gboolean c_str_equal(const char *a, const char *b) {
+	return g_str_equal(a, b);
+}
+
 
 /*** MUTEX ABSTRACTION ***/
 
@@ -242,28 +251,44 @@ INLINE int __debug_rwlock_unlock_w(rwlock_t *m, const char *file, unsigned int l
 
 #endif
 
-INLINE void rtpe_auto_cleanup_mutex(mutex_t **m) {
-	mutex_unlock(*m);
+typedef mutex_t mutex_lock_t;
+typedef rwlock_t rwlock_w_lock_t;
+typedef rwlock_t rwlock_r_lock_t;
+
+INLINE void mutex_ptr_unlock(mutex_lock_t *m) {
+	mutex_unlock(m);
 }
-INLINE void rtpe_auto_cleanup_rwlock_r(rwlock_t **m) {
-	rwlock_unlock_r(*m);
+INLINE mutex_lock_t *mutex_auto_lock(mutex_t *m) {
+	mutex_lock(m);
+	return m;
 }
-INLINE void rtpe_auto_cleanup_rwlock_w(rwlock_t **m) {
-	rwlock_unlock_w(*m);
+INLINE void rwlock_ptr_unlock_r(rwlock_r_lock_t *m) {
+	rwlock_unlock_r(m);
 }
+INLINE rwlock_r_lock_t *rwlock_auto_lock_r(rwlock_t *m) {
+	rwlock_lock_r(m);
+	return m;
+}
+INLINE void rwlock_ptr_unlock_w(rwlock_w_lock_t *m) {
+	rwlock_unlock_w(m);
+}
+INLINE rwlock_w_lock_t *rwlock_auto_lock_w(rwlock_t *m) {
+	rwlock_lock_w(m);
+	return m;
+}
+
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(mutex_lock_t, mutex_ptr_unlock)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(rwlock_r_lock_t, rwlock_ptr_unlock_w)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(rwlock_w_lock_t, rwlock_ptr_unlock_r)
 
 #define CONCAT2(a, b) a ## b
 #define CONCAT(a, b) CONCAT2(a, b)
-#define LOCK(m) AUTO_CLEANUP(mutex_t *CONCAT(__auto_lock_, __COUNTER__), rtpe_auto_cleanup_mutex) \
-	__attribute__((unused)) = m; \
-	mutex_lock(m)
-#define RWLOCK_R(m) AUTO_CLEANUP(rwlock_t *CONCAT(__auto_lock_, __COUNTER__), rtpe_auto_cleanup_rwlock_r) \
-	__attribute__((unused)) = m; \
-	rwlock_lock_r(m)
-#define RWLOCK_W(m) AUTO_CLEANUP(rwlock_t *CONCAT(__auto_lock_, __COUNTER__), rtpe_auto_cleanup_rwlock_w) \
-	__attribute__((unused)) = m; \
-	rwlock_lock_w(m)
-
+#define LOCK(m) g_autoptr(mutex_lock_t) CONCAT(__auto_lock_, __COUNTER__) \
+		__attribute__((unused)) = mutex_auto_lock(m)
+#define RWLOCK_R(m) g_autoptr(rwlock_r_lock_t) CONCAT(__auto_lock_, __COUNTER__) \
+		__attribute__((unused)) = rwlock_auto_lock_r(m)
+#define RWLOCK_W(m) g_autoptr(rwlock_w_lock_t) CONCAT(__auto_lock_, __COUNTER__) \
+		__attribute__((unused)) = rwlock_auto_lock_w(m)
 
 
 
@@ -343,33 +368,35 @@ INLINE void g_tree_clear(GTree *t) {
 		g_tree_remove(t, k);
 	}
 }
-INLINE void g_string_free_true(GString *s) {
-	g_string_free(s, TRUE);
-}
-INLINE void __g_string_free(GString **s) {
-	g_string_free(*s, TRUE);
-}
-INLINE void __g_hash_table_destroy(GHashTable **s) {
-	g_hash_table_destroy(*s);
-}
 
 
-int g_tree_find_first_cmp(void *, void *, void *);
-int g_tree_find_all_cmp(void *, void *, void *);
+int rtpe_tree_find_first_cmp(void *, void *, void *);
+int rtpe_tree_find_all_cmp(void *, void *, void *);
+
+struct rtpe_g_tree_find_helper {
+	GEqualFunc func;
+	void *data;
+	union {
+		GQueue *out_q;
+		void *out_p;
+	};
+};
+
 INLINE void *g_tree_find_first(GTree *t, GEqualFunc f, void *data) {
-	void *p[3];
-	p[0] = data;
-	p[1] = f;
-	p[2] = NULL;
-	g_tree_foreach(t, g_tree_find_first_cmp, p);
-	return p[2];
+	struct rtpe_g_tree_find_helper h = {
+		.func = f,
+		.data = data,
+	};
+	g_tree_foreach(t, rtpe_tree_find_first_cmp, &h);
+	return h.out_p;
 }
 INLINE void g_tree_find_all(GQueue *out, GTree *t, GEqualFunc f, void *data) {
-	void *p[3];
-	p[0] = data;
-	p[1] = f;
-	p[2] = out;
-	g_tree_foreach(t, g_tree_find_all_cmp, p);
+	struct rtpe_g_tree_find_helper h = {
+		.func = f,
+		.data = data,
+		.out_q = out,
+	};
+	g_tree_foreach(t, rtpe_tree_find_all_cmp, &h);
 }
 INLINE void g_tree_get_values(GQueue *out, GTree *t) {
 	g_tree_find_all(out, t, NULL, NULL);
@@ -445,6 +472,13 @@ INLINE pid_t gettid(void) {
 }
 #endif
 
+INLINE unsigned int int64_hash(const uint64_t *s) {
+	return g_int64_hash(s);
+}
+INLINE gboolean int64_eq(const uint64_t *a, const uint64_t *b) {
+	return *a == *b;
+}
+
 
 
 /*** TAINT FUNCTIONS ***/
@@ -466,4 +500,160 @@ taint_func(random, "use ssl_random() instead");
 taint_func(srandom, "use rtpe_ssl_init() instead");
 
 
+/*** ATOMIC64 ***/
+
+typedef struct {
+	uint64_t a;
+} atomic64;
+
+INLINE uint64_t atomic64_get(const atomic64 *u) {
+	return __atomic_load_n(&u->a, __ATOMIC_SEQ_CST);
+}
+INLINE uint64_t atomic64_get_na(const atomic64 *u) {
+	return __atomic_load_n(&u->a, __ATOMIC_RELAXED);
+}
+INLINE void atomic64_set(atomic64 *u, uint64_t a) {
+	__atomic_store_n(&u->a, a, __ATOMIC_SEQ_CST);
+}
+INLINE gboolean atomic64_set_if(atomic64 *u, uint64_t a, uint64_t i) {
+	return __atomic_compare_exchange_n(&u->a, &i, a, false, __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST);
+}
+INLINE void atomic64_set_na(atomic64 *u, uint64_t a) {
+	__atomic_store_n(&u->a, a, __ATOMIC_RELAXED);
+}
+INLINE uint64_t atomic64_add(atomic64 *u, uint64_t a) {
+	return __atomic_fetch_add(&u->a, a, __ATOMIC_SEQ_CST);
+}
+INLINE uint64_t atomic64_add_na(atomic64 *u, uint64_t a) {
+	return __atomic_fetch_add(&u->a, a, __ATOMIC_RELAXED);
+}
+INLINE uint64_t atomic64_get_set(atomic64 *u, uint64_t a) {
+	uint64_t old;
+	do {
+		old = atomic64_get(u);
+		if (__atomic_compare_exchange_n(&u->a, &old, a, false, __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST))
+			return old;
+	} while (1);
+}
+INLINE uint64_t atomic64_or(atomic64 *u, uint64_t a) {
+	return __atomic_fetch_or(&u->a, a, __ATOMIC_SEQ_CST);
+}
+INLINE uint64_t atomic64_and(atomic64 *u, uint64_t a) {
+	return __atomic_fetch_and(&u->a, a, __ATOMIC_SEQ_CST);
+}
+
+INLINE uint64_t atomic64_inc(atomic64 *u) {
+	return atomic64_add(u, 1);
+}
+INLINE uint64_t atomic64_inc_na(atomic64 *u) {
+	return atomic64_add_na(u, 1);
+}
+INLINE uint64_t atomic64_dec(atomic64 *u) {
+	return atomic64_add(u, -1);
+}
+INLINE void atomic64_local_copy_zero(atomic64 *dst, atomic64 *src) {
+	atomic64_set_na(dst, atomic64_get_set(src, 0));
+}
+#define atomic64_local_copy_zero_struct(d, s, member) \
+	atomic64_local_copy_zero(&((d)->member), &((s)->member))
+
+INLINE void atomic64_min(atomic64 *min, uint64_t val) {
+	do {
+		uint64_t old = atomic64_get_na(min);
+		if (old && old <= val)
+			break;
+		if (atomic64_set_if(min, val, old))
+			break;
+	} while (1);
+}
+INLINE void atomic64_max(atomic64 *max, uint64_t val) {
+	do {
+		uint64_t old = atomic64_get_na(max);
+		if (old && old >= val)
+			break;
+		if (atomic64_set_if(max, val, old))
+			break;
+	} while (1);
+}
+
+INLINE void atomic64_calc_rate_from_diff(long long run_diff_us, uint64_t diff, atomic64 *rate_var) {
+	atomic64_set_na(rate_var, run_diff_us ? diff * 1000000LL / run_diff_us : 0);
+}
+INLINE void atomic64_calc_rate(const atomic64 *ax_var, long long run_diff_us,
+		atomic64 *intv_var, atomic64 *rate_var)
+{
+	uint64_t ax = atomic64_get_na(ax_var);
+	uint64_t old_intv = atomic64_get_na(intv_var);
+	atomic64_set_na(intv_var, ax);
+	atomic64_calc_rate_from_diff(run_diff_us, ax - old_intv, rate_var);
+}
+INLINE void atomic64_calc_diff(const atomic64 *ax_var, atomic64 *intv_var, atomic64 *diff_var) {
+	uint64_t ax = atomic64_get_na(ax_var);
+	uint64_t old_intv = atomic64_get_na(intv_var);
+	atomic64_set_na(intv_var, ax);
+	atomic64_set_na(diff_var, ax - old_intv);
+}
+INLINE void atomic64_mina(atomic64 *min, atomic64 *inp) {
+	atomic64_min(min, atomic64_get_na(inp));
+}
+INLINE void atomic64_maxa(atomic64 *max, atomic64 *inp) {
+	atomic64_max(max, atomic64_get_na(inp));
+}
+INLINE double atomic64_div(const atomic64 *n, const atomic64 *d) {
+	int64_t dd = atomic64_get_na(d);
+	if (!dd)
+		return 0.;
+	return (double) atomic64_get_na(n) / (double) dd;
+}
+
+#define atomic_get_na(x) __atomic_load_n(x, __ATOMIC_RELAXED)
+#define atomic_set_na(x,y) __atomic_store_n(x, y, __ATOMIC_RELAXED)
+#define atomic_inc_na(x) __atomic_fetch_add(x, 1, __ATOMIC_RELAXED);
+
+
+/*** ATOMIC BITFIELD OPERATIONS ***/
+
+/* checks if at least one of the flags is set */
+INLINE bool bf_isset(const atomic64 *u, const uint64_t f) {
+	if ((atomic64_get(u) & f))
+		return true;
+	return false;
+}
+/* checks if all of the flags are set */
+INLINE bool bf_areset(const atomic64 *u, const uint64_t f) {
+	if ((atomic64_get(u) & f) == f)
+		return true;
+	return false;
+}
+/* returns true if at least one of the flags was set already */
+INLINE bool bf_set(atomic64 *u, const uint64_t f) {
+	return (atomic64_or(u, f) & f) ? true : false;
+}
+/* returns true if at least one of the flags was set */
+INLINE bool bf_clear(atomic64 *u, const uint64_t f) {
+	return (atomic64_and(u, ~f) & f) ? true : false;
+}
+INLINE void bf_set_clear(atomic64 *u, const uint64_t f, bool cond) {
+	if (cond)
+		bf_set(u, f);
+	else
+		bf_clear(u, f);
+}
+/* works only for single flags */
+INLINE void bf_copy(atomic64 *u, const uint64_t f,
+		const atomic64 *s, const uint64_t g)
+{
+	bf_set_clear(u, f, bf_isset(s, g));
+}
+/* works for multiple flags */
+INLINE void bf_copy_same(atomic64 *u, const atomic64 *s, const uint64_t g) {
+	unsigned int old, set, clear;
+	old = atomic64_get(s);
+	set = old & g;
+	clear = ~old & g;
+	bf_set(u, set);
+	bf_clear(u, clear);
+}
+
+
 #endif
