@@ -45,10 +45,11 @@ struct stream {
 	unsigned long long output_ts;
 	decoder_t *decoder;
 	encoder_t *encoder;
-	codec_chain_t *chain;
+	codec_cc_t *chain;
 	struct testparams in_params;
 	struct testparams out_params;
 	uint fixture_idx;
+	long long encoding_start;
 
 	uint dump_count;
 	AVFormatContext *fmtctx;
@@ -65,7 +66,7 @@ struct stats {
 };
 
 struct stats_sample {
-	struct timeval tv; // last time stats were sampled
+	long long ts; // last time stats were sampled
 	struct stats stats; // last sampled stats
 };
 
@@ -99,10 +100,18 @@ struct thread_freq_stats {
 	struct freq_stats stats;
 };
 
+struct delay_stats {
+	long long max_actual;
+	long long max_allowed;
+	uint slots;
+	uint *counts;
+};
+
 
 typedef void render_fn(const struct stats *stats, int line, int x, int breadth, int width,
 		int color,
 		const char *titlefmt, ...);
+typedef void delay_fn(const struct delay_stats *stats, int line, int x, int breadth, int width);
 
 
 
@@ -197,16 +206,27 @@ static WINDOW *popup;
 
 static long long ptime = 20000; // us TODO: support different ptimes
 
+static mutex_t delay_stats_lock = MUTEX_STATIC_INIT;
+static struct delay_stats delay_stats;
+
 
 
 static render_fn usage_bar;
 static render_fn time_bar;
+static render_fn no_bar;
+
+static delay_fn delay_bar;
+static delay_fn no_delay;
 
 static render_fn *output_fn = usage_bar; // startup default
+static delay_fn *delay_out_fn = no_delay; // startup default
 static bool do_cpu_stats = false;
 static bool do_thread_stats = false;
 
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(FILE, fclose)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(DIR, closedir)
+
 
 
 static pthread_t thread_new(const char *name, void *(*fn)(void *), void *p) {
@@ -225,6 +245,11 @@ static pthread_t thread_new(const char *name, void *(*fn)(void *), void *p) {
 static inline long long us_ticks_scale(long long val) {
 	return val * ticks_per_sec / 1000000;
 }
+static inline long long now_us(void) {
+	struct timeval now;
+	gettimeofday(&now, NULL);
+	return timeval_us(&now);
+}
 
 
 // stream is locked
@@ -246,6 +271,19 @@ static int got_packet_pkt(struct stream *s, AVPacket *pkt) {
 	ssize_t ret = write(s->output_fd, pkt->data, pkt->size);
 	(void)ret;
 
+	long long now = now_us();
+	long long diff = now - s->encoding_start;
+
+	{
+		LOCK(&delay_stats_lock);
+		if (delay_stats.max_actual < diff)
+			delay_stats.max_actual = diff;
+		if (delay_stats.max_allowed && diff < delay_stats.max_allowed) {
+			uint slot = diff * delay_stats.slots / delay_stats.max_allowed;
+			delay_stats.counts[slot]++;
+		}
+	}
+
 	if (s->fmtctx) {
 		// mkv uses millisecond timestamps
 		pkt->pts = pkt->dts = av_rescale(pkt->pts, 1000, out_params.clock_rate);
@@ -286,17 +324,15 @@ static void *worker(void *p) {
 		LOCK(&other_threads_lock);
 		g_hash_table_insert(worker_threads, GINT_TO_POINTER(worker_self->pid), NULL);
 	}
-	poller_loop2(rtpe_poller);
+	poller_loop(rtpe_poller);
 	return NULL;
 }
 
-static void readable(int fd, void *o, uintptr_t x) {
+static void readable(int fd, void *o) {
 	struct stream *s = o;
+	obj_hold(s);
 
-	struct timeval start;
-	gettimeofday(&start, NULL);
-
-	LOCK(&s->lock);
+	long long start = now_us();
 
 	static const uint64_t max_iters = 10; // hard upper limit for iterations
 	uint64_t total_iters = 0;
@@ -320,6 +356,10 @@ static void readable(int fd, void *o, uintptr_t x) {
 			break; // bail
 
 		while (exp) {
+			LOCK(&s->lock);
+
+			s->encoding_start = start;
+
 			AVPacket *data = s->in_params.fixture->pdata[s->fixture_idx++];
 			if (s->fixture_idx >= s->in_params.fixture->len)
 				s->fixture_idx = 0;
@@ -330,8 +370,11 @@ static void readable(int fd, void *o, uintptr_t x) {
 			if (!s->chain)
 				decoder_input_data(s->decoder, &frame, s->input_ts, got_frame, s, NULL);
 			else {
-				AVPacket *pkt = codec_chain_input_data(s->chain, &frame, s->input_ts);
-				got_packet_pkt(s, pkt);
+				AVPacket *pkt = codec_cc_input_data(s->chain, &frame, s->input_ts, s, NULL, NULL);
+				if (pkt)
+					got_packet_pkt(s, pkt);
+				else
+					mutex_lock(&s->lock); // was unlocked by async_init
 			}
 
 			s->input_ts += data->duration;
@@ -340,14 +383,15 @@ static void readable(int fd, void *o, uintptr_t x) {
 		}
 	}
 
-	struct timeval end;
-	gettimeofday(&end, NULL);
+	obj_put(s);
+
+	long long end = now_us();
 
 	LOCK(&worker_self->comput_lock);
-	worker_self->comput += timeval_diff(&end, &start);
+	worker_self->comput += end - start;
 }
 
-static void closed(int fd, void *o, uintptr_t x) {
+static void closed(int fd, void *o) {
 	abort();
 }
 
@@ -390,7 +434,6 @@ static void kill_threads(uint num) {
 
 static void stream_free(void *p) {
 	struct stream *s = p;
-	close(s->timer_fd);
 	close(s->output_fd);
 	dump_close(s);
 	if (s->encoder)
@@ -401,6 +444,23 @@ static void stream_free(void *p) {
 }
 
 
+static void *async_init(void *x, void *y, void *z) {
+	struct stream *s = x;
+	// unlock in case the chain is busy and this blocks, so that whoever keeps the chain
+	// busy can lock the stream once the result is in
+	mutex_unlock(&s->lock);
+	return obj_hold(s);
+}
+static void async_finish(AVPacket *pkt, void *async_cb_obj) {
+	struct stream *s = async_cb_obj;
+	{
+		LOCK(&s->lock);
+		got_packet_pkt(s, pkt);
+	}
+	obj_put(s);
+	av_packet_free(&pkt);
+}
+
 static void new_stream_params(
 		const codec_def_t *in_def,
 		const struct testparams *inprm,
@@ -444,7 +504,7 @@ static void new_stream_params(
 
 	format_t actual_enc_format;
 
-	s->chain = codec_chain_new(in_def, &dec_format, out_def, &enc_format, bitrate, 20);
+	s->chain = codec_cc_new(in_def, &dec_format, out_def, &enc_format, bitrate, 20, async_init, async_finish);
 
 	if (!s->chain) {
 		s->encoder = encoder_new();
@@ -480,8 +540,8 @@ static void new_stream_params(
 		.closed = closed,
 	};
 
-	res = poller_add_item(rtpe_poller, &pi);
-	assert(res == 0);
+	bool ok = poller_add_item(rtpe_poller, &pi);
+	assert(ok == true);
 
 	LOCK(&streams_lock);
 	g_ptr_array_add(streams, s);
@@ -526,6 +586,7 @@ static void del_stream(void) {
 		return;
 
 	poller_del_item(rtpe_poller, s->timer_fd);
+	s->timer_fd = -1;
 
 	obj_put(s);
 }
@@ -595,7 +656,7 @@ static void show_popup(const char *fmt, ...) {
 	str st = STR_INIT(s);
 	str token;
 	uint llen = 0;
-	while (str_token_sep(&token, &st, '\n') == 0) {
+	while (str_token_sep(&token, &st, '\n')) {
 		g_queue_push_tail(&lines, str_dup(&token));
 		llen = MAX(token.len, llen);
 	}
@@ -655,7 +716,7 @@ static char *start_dump_stream(struct stream *s, const char *suffix) {
 		s->fmtctx = avformat_alloc_context();
 		if (!s->fmtctx)
 			goto out;
-		AUTO_CLEANUP(char *fn, free_gbuf)
+		g_autoptr(char) fn
 			= g_strdup_printf("stream-dump-%llu%s.mkv",
 					(long long unsigned) time(NULL),
 					suffix ?: "");
@@ -845,10 +906,17 @@ static void *do_input(void *p) {
 
 			case '1':
 				output_fn = usage_bar;
+				delay_out_fn = no_delay;
 				break;
 
 			case '2':
 				output_fn = time_bar;
+				delay_out_fn = no_delay;
+				break;
+
+			case '3':
+				output_fn = no_bar;
+				delay_out_fn = delay_bar;
 				break;
 
 			case 'o':
@@ -990,16 +1058,9 @@ static void time_bar(const struct stats *stats, int line, int x, int breadth, in
 }
 
 
-static void fclose_p(FILE **p) {
-	if (!*p)
-		return;
-	fclose(*p);
-}
-
-static void closedir_p(DIR **p) {
-	if (!*p)
-		return;
-	closedir(*p);
+static void no_bar(const struct stats *stats, int line, int x, int breadth, int width, int color,
+		const char *titlefmt, ...)
+{
 }
 
 
@@ -1009,14 +1070,13 @@ static bool thread_collect(pid_t pid, struct stats *outp, struct stats_sample *s
 	if (!pid)
 		return false;
 
-	AUTO_CLEANUP(char *fn, free_gbuf)
+	g_autoptr(char) fn
 		= g_strdup_printf("/proc/%i/task/%i/stat", (int) pid, (int) pid);
-	AUTO_CLEANUP(FILE *fp, fclose_p) = fopen(fn, "r");
+	g_autoptr(FILE) fp = fopen(fn, "r");
 	if (!fp)
 		return false;
 
-	struct timeval now;
-	gettimeofday(&now, NULL);
+	long long now = now_us();
 
 	long long utime, stime;
 	char comm[COMM_SIZE];
@@ -1025,13 +1085,13 @@ static bool thread_collect(pid_t pid, struct stats *outp, struct stats_sample *s
 	if (rets != 3)
 		return false;
 
-	if (sample->tv.tv_sec) {
-		outp->iv = us_ticks_scale(timeval_diff(&now, &sample->tv));
+	if (sample->ts) {
+		outp->iv = us_ticks_scale(now - sample->ts);
 		outp->ucpu = utime - sample->stats.ucpu;
 		outp->scpu = stime - sample->stats.scpu;
 	}
 
-	sample->tv = now;
+	sample->ts = now;
 	sample->stats.ucpu = utime;
 	sample->stats.scpu = stime;
 
@@ -1108,14 +1168,25 @@ static void worker_stats(struct worker *w, int idx, int starty, int height, int
 }
 
 
-static bool cpu_collect(GQueue *outp, struct stats *totals) {
-	AUTO_CLEANUP(FILE *fp, fclose_p) = fopen("/proc/stat", "r");
+TYPED_GQUEUE(stats, struct stats)
+
+static void stats_queue_free(stats_q *q) {
+	struct stats *sp;
+	while ((sp = t_queue_pop_head(q)))
+		g_slice_free1(sizeof(*sp), sp);
+	t_queue_free(q);
+}
+
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(stats_q, stats_queue_free)
+
+
+static bool cpu_collect(stats_q *outp, struct stats *totals) {
+	g_autoptr(FILE) fp = fopen("/proc/stat", "r");
 	if (!fp)
 		return false;
 
 	while (!feof(fp)) {
-		struct timeval now;
-		gettimeofday(&now, NULL);
+		long long now = now_us();
 
 		char cpu[7];
 		long long utime, nice, stime;
@@ -1132,13 +1203,13 @@ static bool cpu_collect(GQueue *outp, struct stats *totals) {
 
 		struct stats stats = {0};
 
-		if (cpu_stats[idx].tv.tv_sec) {
-			stats.iv = us_ticks_scale(timeval_diff(&now, &cpu_stats[idx].tv));
+		if (cpu_stats[idx].ts) {
+			stats.iv = us_ticks_scale(now - cpu_stats[idx].ts);
 			stats.ucpu = utime - cpu_stats[idx].stats.ucpu;
 			stats.scpu = stime - cpu_stats[idx].stats.scpu;
 		}
 
-		cpu_stats[idx].tv = now;
+		cpu_stats[idx].ts = now;
 		cpu_stats[idx].stats.ucpu = utime;
 		cpu_stats[idx].stats.scpu = stime;
 
@@ -1151,7 +1222,7 @@ static bool cpu_collect(GQueue *outp, struct stats *totals) {
 		if (outp) {
 			struct stats *sp = g_slice_alloc(sizeof(*sp));
 			*sp = stats;
-			g_queue_push_tail(outp, sp);
+			t_queue_push_tail(outp, sp);
 		}
 	}
 
@@ -1159,18 +1230,8 @@ static bool cpu_collect(GQueue *outp, struct stats *totals) {
 }
 
 
-static void stats_queue_free(GQueue **q) {
-	if (!*q)
-		return;
-	struct stats *sp;
-	while ((sp = g_queue_pop_head(*q)))
-		g_slice_free1(sizeof(*sp), sp);
-	g_queue_free(*q);
-}
-
-
 static int cpu_collect_stats(const bool do_output, int starty, int maxy, int maxx, struct stats *totals) {
-	AUTO_CLEANUP(GQueue *stats, stats_queue_free) = do_output ? g_queue_new() : NULL;
+	g_autoptr(stats_q) stats = do_output ? stats_q_new() : NULL;
 
 	if (!cpu_collect(stats, totals))
 		return starty;
@@ -1185,7 +1246,7 @@ static int cpu_collect_stats(const bool do_output, int starty, int maxy, int max
 	int x = 0;
 
 	uint idx = 0;
-	for (GList *l = stats->head; l; l = l->next) {
+	for (__auto_type l = stats->head; l; l = l->next) {
 		struct stats *sp = l->data;
 		grid_line(&y, &x, starty, height, maxx);
 		if (y < maxy)
@@ -1197,6 +1258,42 @@ static int cpu_collect_stats(const bool do_output, int starty, int maxy, int max
 }
 
 
+static void delay_stats_collect(struct delay_stats *local, uint slots, long long max_allowed) {
+	{
+		// copy out and reset to zero
+		LOCK(&delay_stats_lock);
+		*local = delay_stats;
+
+		delay_stats = (struct delay_stats) {0};
+		delay_stats.slots = slots;
+		delay_stats.counts = g_new0(__typeof__(*delay_stats.counts), delay_stats.slots);
+		delay_stats.max_allowed = max_allowed;
+	}
+}
+
+static void delay_stats_free(struct delay_stats *local) {
+	g_free(local->counts);
+}
+
+
+static void no_delay(const struct delay_stats *stats, int line, int x, int breadth, int width) {
+}
+
+static void delay_bar(const struct delay_stats *stats, int line, int x, int breadth, int width) {
+	if (!stats->slots)
+		return;
+
+	uint per_slot = stats->max_allowed / stats->slots;
+
+	for (uint i = 0; i < stats->slots; i++) {
+		move(line, x);
+		uint start = i * stats->max_allowed / stats->slots;
+		printw("%3.1f ms - %3.1f ms: %u", start / 1000., (start + per_slot) / 1000., stats->counts[i]);
+		line++;
+	}
+}
+
+
 static void other_thread_free(struct other_thread *thr) {
 	g_slice_free1(sizeof(*thr), thr);
 }
@@ -1214,8 +1311,8 @@ static int pid_compare(const void *a, const void *b) {
 static int other_threads_collect(const bool do_output, int starty, int maxy, int maxx,
 		struct stats *totals)
 {
-	AUTO_CLEANUP(char *dn, free_gbuf) = g_strdup_printf("/proc/%u/task", getpid());
-	AUTO_CLEANUP(DIR *dp, closedir_p) = opendir(dn);
+	g_autoptr(char) dn = g_strdup_printf("/proc/%u/task", getpid());
+	g_autoptr(DIR) dp = opendir(dn);
 	if (!dp)
 		return starty;
 
@@ -1365,6 +1462,10 @@ static void *do_stats(void *p) {
 				line += 2;
 			}
 
+			// collect delay stats
+			struct delay_stats delay_stats_local;
+			delay_stats_collect(&delay_stats_local, maxy - 3, 20000);
+
 			// worker thread stats
 			totals_line = line;
 			struct stats worker_totals = {0};
@@ -1404,7 +1505,11 @@ static void *do_stats(void *p) {
 
 			output_fn(&worker_totals, totals_line, 0, breadth, maxx, SUMMARY_COLOR, "Threads:");
 
+			delay_out_fn(&delay_stats_local, totals_line, 0, breadth, maxx);
+
 			refresh_all();
+
+			delay_stats_free(&delay_stats_local);
 		}
 
 		thread_cancel_enable();
@@ -1423,7 +1528,7 @@ static char *fixture_path_file(const char *base_fn) {
 
 static void fixture_read_avio(GPtrArray *fixture, struct testparams *prm) {
 	AVFormatContext *fctx = NULL;
-	AUTO_CLEANUP(char *fn, free_gbuf) = fixture_path_file(prm->file);
+	g_autoptr(char) fn = fixture_path_file(prm->file);
 	int ret = avformat_open_input(&fctx, fn, NULL, NULL);
 	if (ret < 0)
 		die("Failed to open input fixture");
@@ -1452,7 +1557,7 @@ static void fixture_read_avio(GPtrArray *fixture, struct testparams *prm) {
 
 
 static void fixture_read_raw(GPtrArray *fixture, struct testparams *prm) {
-	AUTO_CLEANUP(char *fn, free_gbuf) = fixture_path_file(prm->file);
+	g_autoptr(char) fn = fixture_path_file(prm->file);
 	FILE *fp = fopen(fn, "r");
 	if (!fp)
 		die("Failed to open input fixture");
@@ -1655,7 +1760,7 @@ static void *cpu_freq_monitor(void *p) {
 		struct freq_stats iter_stats = {0};
 
 		{
-			AUTO_CLEANUP(DIR *dp, closedir_p) = opendir("/sys/devices/system/cpu/cpufreq");
+			g_autoptr(DIR) dp = opendir("/sys/devices/system/cpu/cpufreq");
 			if (!dp)
 				break; // bail out
 
@@ -1665,10 +1770,10 @@ static void *cpu_freq_monitor(void *p) {
 				if (strncmp(ent->d_name, "policy", 6) != 0)
 					continue; // skip
 
-				AUTO_CLEANUP(char *fn, free_gbuf)
+				g_autoptr(char) fn
 					= g_strdup_printf("/sys/devices/system/cpu/cpufreq/%s/scaling_cur_freq",
 							ent->d_name);
-				AUTO_CLEANUP(FILE *fp, fclose_p) = fopen(fn, "r");
+				g_autoptr(FILE) fp  = fopen(fn, "r");
 				if (!fp)
 					continue; // ignore
 
