@@ -59,6 +59,7 @@ char *forward_to = NULL;
 static char *tls_send_to = NULL;
 endpoint_t tls_send_to_ep;
 int tls_resample = 8000;
+bool tls_disable = false;
 char *notify_uri;
 gboolean notify_post;
 gboolean notify_nverify;
@@ -66,6 +67,7 @@ int notify_threads = 5;
 int notify_retries = 10;
 gboolean notify_record;
 gboolean notify_purge;
+gboolean mix_output_per_media = 0;
 
 static GQueue threads = G_QUEUE_INIT; // only accessed from main thread
 
@@ -183,12 +185,13 @@ static mode_t chmod_parse(const char *s) {
 
 
 static void options(int *argc, char ***argv) {
-	AUTO_CLEANUP_GBUF(os_str);
-	AUTO_CLEANUP_GBUF(chmod_mode);
-	AUTO_CLEANUP_GBUF(chmod_dir_mode);
-	AUTO_CLEANUP_GBUF(user_uid);
-	AUTO_CLEANUP_GBUF(group_gid);
-	AUTO_CLEANUP_GBUF(mix_method_str);
+	g_autoptr(char) os_str = NULL;
+	g_autoptr(char) chmod_mode = NULL;
+	g_autoptr(char) chmod_dir_mode = NULL;
+	g_autoptr(char) user_uid = NULL;
+	g_autoptr(char) group_gid = NULL;
+	g_autoptr(char) mix_method_str = NULL;
+	g_autoptr(char) tcp_send_to = NULL;
 
 	GOptionEntry e[] = {
 		{ "table",		't', 0, G_OPTION_ARG_INT,	&ktable,	"Kernel table rtpengine uses",		"INT"		},
@@ -214,13 +217,16 @@ static void options(int *argc, char ***argv) {
 		{ "mysql-pass",		0,   0,	G_OPTION_ARG_STRING,	&c_mysql_pass,	"MySQL connection credentials",		"PASSWORD"	},
 		{ "mysql-db",		0,   0,	G_OPTION_ARG_STRING,	&c_mysql_db,	"MySQL database name",			"STRING"	},
 		{ "forward-to", 	0,   0, G_OPTION_ARG_STRING,	&forward_to,	"Where to forward to (unix socket)",	"PATH"		},
+		{ "tcp-send-to", 	0,   0, G_OPTION_ARG_STRING,	&tcp_send_to,	"Where to send to (TCP destination)",	"IP:PORT"	},
 		{ "tls-send-to", 	0,   0, G_OPTION_ARG_STRING,	&tls_send_to,	"Where to send to (TLS destination)",	"IP:PORT"	},
-		{ "tls-resample", 	0,   0, G_OPTION_ARG_INT,	&tls_resample,	"Sampling rate for TLS PCM output",	"INT"		},
+		{ "tcp-resample", 	0,   0, G_OPTION_ARG_INT,	&tls_resample,	"Sampling rate for TCP/TLS PCM output",	"INT"		},
+		{ "tls-resample", 	0,   0, G_OPTION_ARG_INT,	&tls_resample,	"Sampling rate for TCP/TLS PCM output",	"INT"		},
 		{ "notify-uri", 	0,   0, G_OPTION_ARG_STRING,	&notify_uri,	"Notify destination for finished outputs","URI"		},
 		{ "notify-post", 	0,   0, G_OPTION_ARG_NONE,	&notify_post,	"Use POST instead of GET",		NULL		},
 		{ "notify-no-verify", 	0,   0, G_OPTION_ARG_NONE,	&notify_nverify,"Don't verify HTTPS peer certificate",	NULL		},
 		{ "notify-concurrency",	0,   0, G_OPTION_ARG_INT,	&notify_threads,"How many simultaneous requests",	"INT"		},
 		{ "notify-retries",	0,   0, G_OPTION_ARG_INT,	&notify_retries,"How many times to retry failed requesets","INT"	},
+		{ "output-mixed-per-media",0,0,	G_OPTION_ARG_NONE,	&mix_output_per_media,"Mix participating sources into a single output", NULL },
 #if CURL_AT_LEAST_VERSION(7,56,0)
 		{ "notify-record", 	0,   0, G_OPTION_ARG_NONE,	&notify_record, "Also attach recorded file to request", NULL		},
 		{ "notify-purge", 	0,   0, G_OPTION_ARG_NONE,	&notify_purge,	"Remove the local file if notify success", NULL		},
@@ -241,9 +247,16 @@ static void options(int *argc, char ***argv) {
 	if (output_format == NULL)
 		output_format = g_strdup("wav");
 
+	if (tcp_send_to) {
+		if (tls_send_to)
+			die("Cannot have both 'tcp-send-to' and 'tls-send-to' active at the same time");
+		tls_send_to = tcp_send_to;
+		tls_disable = true;
+	}
+
 	if (tls_send_to) {
 		if (endpoint_parse_any_getaddrinfo_full(&tls_send_to_ep, tls_send_to))
-			die("Failed to parse 'tls-send-to' option");
+			die("Failed to parse 'tcp-send-to' or 'tls-send-to' option");
 	}
 
 	if (!strcmp(output_format, "none")) {
@@ -317,7 +330,7 @@ static void options(int *argc, char ***argv) {
 		num_threads = num_cpu_cores(8);
 
 	if (!output_pattern)
-		output_pattern = g_strdup("%c-%t");
+		output_pattern = g_strdup("%c-%r-%t");
 	if (!strstr(output_pattern, "%c"))
 		die("Invalid output pattern '%s' (no '%%c' format present)", output_pattern);
 	if (!strstr(output_pattern, "%t"))
