@@ -83,8 +83,15 @@ static void mix_input_reset(mix_t *mix, unsigned int idx) {
 }
 
 
-unsigned int mix_get_index(mix_t *mix, void *ptr) {
+unsigned int mix_get_index(mix_t *mix, void *ptr, unsigned int media_sdp_id) {
 	unsigned int next = mix->next_idx++;
+	if (mix_output_per_media) {
+		next = media_sdp_id;
+		if (next >= mix_num_inputs) {
+			ilog(LOG_WARNING, "Error with mix_output_per_media sdp_label next %i is bigger than mix_num_inputs %i", next, mix_num_inputs );
+		}
+	}
+
 	if (next < mix_num_inputs) {
 		// must be unused
 		mix->input_ref[next] = ptr;
@@ -182,6 +189,8 @@ int mix_config(mix_t *mix, const format_t *format) {
 		err = "failed to link abuffer to amix";
 		if (avfilter_link(mix->src_ctxs[i], 0, mix->amix_ctx, i))
 			goto err;
+
+
 	}
 
 	// sink
@@ -198,6 +207,7 @@ int mix_config(mix_t *mix, const format_t *format) {
 	if (avfilter_link(mix->amix_ctx, 0, mix->sink_ctx, 0))
 		goto err;
 
+
 	// finish up
 	err = "failed to configure filter chain";
 	if (avfilter_graph_config(mix->graph, NULL))
@@ -216,7 +226,7 @@ err:
 }
 
 
-mix_t *mix_new() {
+mix_t *mix_new(void) {
 	mix_t *mix = g_slice_alloc0(sizeof(*mix));
 	format_init(&mix->in_format);
 	format_init(&mix->out_format);
