@@ -300,11 +300,11 @@ $json_exp = {
 		   '2',
 		   '3'
 		 ],
-  'subscriptions-0' => [
-			 '1/0/1/0/0'
+  'media-subscriptions-0' => [
+			 '1/1/0/0'
 		       ],
-  'subscriptions-1' => [
-			 '0/0/1/0/0'
+  'media-subscriptions-1' => [
+			 '0/1/0/0'
 		       ],
   'tag-0' => {
 	       'block_dtmf' => '0',
@@ -558,11 +558,11 @@ $json_exp = {
 		   '2',
 		   '3'
 		 ],
-  'subscriptions-0' => [
-			 '1/0/1/0/0'
+  'media-subscriptions-0' => [
+			 '1/1/0/0'
 		       ],
-  'subscriptions-1' => [
-			 '0/0/1/0/0'
+  'media-subscriptions-1' => [
+			 '0/1/0/0'
 		       ],
   'tag-0' => {
 	       'block_dtmf' => '0',
@@ -825,11 +825,11 @@ $json_exp = {
 		   '2',
 		   '3'
 		 ],
-  'subscriptions-0' => [
-			 '1/0/1/0/0'
+  'media-subscriptions-0' => [
+			 '1/1/0/0'
 		       ],
-  'subscriptions-1' => [
-			 '0/0/1/0/0'
+  'media-subscriptions-1' => [
+			 '0/1/0/0'
 		       ],
   'tag-0' => {
 	       'block_dtmf' => '0',
@@ -1086,11 +1086,11 @@ $json_exp = {
 		   '2',
 		   '3'
 		 ],
-  'subscriptions-0' => [
-			 '1/0/1/0/0'
+  'media-subscriptions-0' => [
+			 '1/1/0/0'
 		       ],
-  'subscriptions-1' => [
-			 '0/0/1/0/0'
+  'media-subscriptions-1' => [
+			 '0/1/0/0'
 		       ],
   'tag-0' => {
 	       'block_dtmf' => '0',
@@ -1530,15 +1530,14 @@ $json_exp = {
                            '6',
                            '7'
                          ],
-          'subscriptions-0' => [
-                                 '1/0/1/0/0'
+          'media-subscriptions-0' => [
+                                 '1/1/0/0'
                                ],
-          'subscriptions-1' => [
-                                 '0/0/1/0/0'
+          'media-subscriptions-1' => [
+                                 '0/1/0/0'
                                ],
-          'subscriptions-2' => [
-                                 '0/0/0/0/0',
-                                 '1/1/0/0/0'
+          'media-subscriptions-2' => [
+                                 '1/0/0/0'
                                ],
           'tag-0' => {
                        'block_dtmf' => '0',
@@ -1572,8 +1571,8 @@ $json_exp = {
 my ($ftr, $ttr, $fts) = subscribe_request('sub to multiple tags',
 	{ 'from-tags' => [ft(), tt()] }, <<SDP);
 v=0
-o=- SDP_VERSION IN IP4 203.0.113.1
-s=RTPE_VERSION
+o=- SDP_VERSION IN IP4 198.51.100.1
+s=tester
 t=0 0
 m=audio PORT RTP/AVP 0 8
 c=IN IP4 203.0.113.1
@@ -1746,7 +1745,7 @@ $json_exp = {
 		   '0',
 		   '1'
 		 ],
-  'subscriptions-0' => [],
+  'media-subscriptions-0' => [],
   'tag-0' => {
 	       'block_dtmf' => '0',
 	       'created' => qr//,
@@ -1983,9 +1982,9 @@ $json_exp = {
 		   '2',
 		   '3'
 		 ],
-  'subscriptions-0' => [],
-  'subscriptions-1' => [
-			 '0/0/0/0/0'
+  'media-subscriptions-0' => [],
+  'media-subscriptions-1' => [
+			 '0/0/0/0'
 		       ],
   'tag-0' => {
 	       'block_dtmf' => '0',
@@ -2224,9 +2223,9 @@ $json_exp = {
 		   '2',
 		   '3'
 		 ],
-  'subscriptions-0' => [],
-  'subscriptions-1' => [
-			 '0/0/0/0/0'
+  'media-subscriptions-0' => [],
+  'media-subscriptions-1' => [
+			 '0/0/0/0'
 		       ],
   'tag-0' => {
 	       'block_dtmf' => '0',
@@ -2555,12 +2554,12 @@ $json_exp = {
 		   '4',
 		   '5'
 		 ],
-  'subscriptions-0' => [],
-  'subscriptions-1' => [
-			 '0/0/0/0/0'
+  'media-subscriptions-0' => [],
+  'media-subscriptions-1' => [
+			 '0/0/0/0'
 		       ],
-  'subscriptions-2' => [
-			 '0/0/0/0/0'
+  'media-subscriptions-2' => [
+			 '0/0/0/0'
 		       ],
   'tag-0' => {
 	       'block_dtmf' => '0',
